import pytest

from django.urls import reverse

users = (
    ('admin', 'admin'),
    ('manager', 'manager'),
    ('user', 'user'),
    ('test', 'test'),
    ('anonymous', None),
)

status_map = {
    'management': {
        'admin': 200, 'manager': 200, 'user': 403, 'test': 403, 'anonymous': 302
    },
    'public_schema': {
        'admin': 200, 'manager': 200, 'user': 200, 'test': 200, 'anonymous': 200
    },
    'internal_schema': {
        'admin': 200, 'manager': 200, 'user': 200, 'test': 200, 'anonymous': 404
    },
    'private_schema': {
        'admin': 404, 'manager': 404, 'user': 404, 'test': 200, 'anonymous': 404
    },
    'public_table': {
        'admin': 200, 'manager': 200, 'user': 200, 'test': 200, 'anonymous': 200
    },
    'internal_table': {
        'admin': 200, 'manager': 200, 'user': 200, 'test': 200, 'anonymous': 404
    },
    'private_table': {
        'admin': 404, 'manager': 404, 'user': 404, 'test': 200, 'anonymous': 404
    }
}


@pytest.mark.parametrize(('username', 'password'), users)
def test_management(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse('metadata:management')
    response = client.get(url)
    assert response.status_code == status_map['management'][username]


@pytest.mark.parametrize(('username', 'password'), users)
def test_public_schema(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse('metadata:schema', kwargs={
        'schema_name': 'daiquiri_data_obs'
    })
    response = client.get(url)
    assert response.status_code == status_map['public_schema'][username]


@pytest.mark.parametrize(('username', 'password'), users)
def test_internal_schema(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse('metadata:schema', kwargs={
        'schema_name': 'daiquiri_data_sim'
    })
    response = client.get(url)
    assert response.status_code == status_map['internal_schema'][username]


@pytest.mark.parametrize(('username', 'password'), users)
def test_private_schema(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse('metadata:schema', kwargs={
        'schema_name': 'daiquiri_data_test'
    })
    response = client.get(url)
    assert response.status_code == status_map['private_schema'][username]


@pytest.mark.parametrize(('username', 'password'), users)
def test_public_table(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse('metadata:table', kwargs={
        'schema_name': 'daiquiri_data_obs',
        'table_name': 'stars'
    })
    response = client.get(url)
    assert response.status_code == status_map['public_table'][username]


@pytest.mark.parametrize(('username', 'password'), users)
def test_internal_table(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse('metadata:table', kwargs={
        'schema_name': 'daiquiri_data_sim',
        'table_name': 'halos'
    })
    response = client.get(url)
    assert response.status_code == status_map['internal_table'][username]


@pytest.mark.parametrize(('username', 'password'), users)
def test_private_table(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse('metadata:table', kwargs={
        'schema_name': 'daiquiri_data_test',
        'table_name': 'test'
    })
    response = client.get(url)
    assert response.status_code == status_map['private_table'][username]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-10-25 10:11
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_metadata', '0014_rename_pid_to_doi'),
    ]

    operations = [
        migrations.AlterField(
            model_name='database',
            name='doi',
            field=models.CharField(blank=True, max_length=256, null=True, verbose_name='Digital object identifier'),
        ),
        migrations.AlterField(
            model_name='table',
            name='doi',
            field=models.CharField(blank=True, max_length=256, null=True, verbose_name='Digital object identifier'),
        ),
    ]



# Generated by Django 2.2.8 on 2020-01-24 10:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_oai', '0002_resource_type'),
    ]

    operations = [
        migrations.AddField(
            model_name='record',
            name='set_spec',
            field=models.CharField(blank=True, db_index=True, max_length=256, verbose_name='OAI set spec'),
        ),
    ]



# Generated by Django 3.2.12 on 2022-02-04 13:32

from django.db import migrations


def fix_char_fields(model, fields):
    for field in fields:
        if getattr(model, field) is None:
            setattr(model, field, '')


def fix_json_fields(model, fields):
    for field in fields:
        if getattr(model, field) is None:
            setattr(model, field, [])


def run_data_migration(apps, schema_editor):
    Schema = apps.get_model('daiquiri_metadata', 'Schema')
    Table = apps.get_model('daiquiri_metadata', 'Table')
    Column = apps.get_model('daiquiri_metadata', 'Column')
    Function = apps.get_model('daiquiri_metadata', 'Function')

    for schema in Schema.objects.all():
        fix_char_fields(schema, ['title', 'description', 'long_description',
                                 'attribution', 'license', 'doi', 'utype'])
        fix_json_fields(schema, ['related_identifiers', 'creators', 'contributors'])
        schema.save()

    for table in Table.objects.all():
        fix_char_fields(table, ['title', 'description', 'long_description',
                                'attribution', 'license', 'doi', 'utype'])
        fix_json_fields(table, ['related_identifiers', 'creators', 'contributors'])
        table.save()

    for column in Column.objects.all():
        fix_char_fields(column, ['description', 'unit', 'ucd', 'utype', 'datatype'])
        column.save()

    for function in Function.objects.all():
        fix_char_fields(function, ['description'])
        function.save()


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_metadata', '0031_change_order'),
    ]

    operations = [
        migrations.RunPython(run_data_migration)
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-09-15 13:40
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_jobs', '0011_job_client_ip'),
    ]

    operations = [
        migrations.AlterField(
            model_name='job',
            name='job_type',
            field=models.CharField(choices=[('SYNC', 'Syncronous'), ('ASYNC', 'Asyncronous'), ('INTERFACE', 'Interface')], max_length=10),
        ),
    ]



from django.urls import re_path

from .views import FileView, SearchView

app_name = 'files'

urlpatterns = [
    re_path(r'^search/', SearchView.as_view(), name='search'),
    re_path(r'^(?P<file_path>.*)$', FileView.as_view(), name='file'),
]



from rest_framework.permissions import BasePermission


class IsAuthManager(BasePermission):

    def has_permission(self, request, view):
        if not request.user or not request.user.is_authenticated:
            return False

        return request.user.has_perms([
            'daiquiri_auth.view_profile',
            'daiquiri_auth.change_profile'
        ])


class GroupViewPermission(BasePermission):

    def has_permission(self, request, view):
        if not request.user or not request.user.is_authenticated:
            return False

        return request.user.has_perms([
            'auth.view_group',
        ])



from django.apps import AppConfig


class MetadataConfig(AppConfig):
    name = 'daiquiri.metadata'
    label = 'daiquiri_metadata'
    verbose_name = 'Metadata'

    def ready(self):
        from . import handlers  # noqa: F401



# Generated by Django 3.2.12 on 2022-02-11 18:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_oai', '0005_alter_record_resource_id'),
    ]

    operations = [
        migrations.AlterField(
            model_name='record',
            name='resource_id',
            field=models.CharField(blank=True, db_index=True, default='', max_length=256, verbose_name='Resource id'),
            preserve_default=False,
        ),
    ]



import csv
import importlib
import ipaddress
import math
import os
import re
import sys
from collections import defaultdict
from datetime import datetime
from urllib.parse import urlparse
from xml.dom import minidom

from django import forms
from django.conf import settings
from django.contrib.auth.models import Group, Permission, User
from django.contrib.sites.shortcuts import get_current_site
from django.core.mail import EmailMessage, EmailMultiAlternatives
from django.db.models import Q
from django.http import HttpResponse
from django.template import TemplateDoesNotExist
from django.template.defaultfilters import date
from django.template.loader import render_to_string
from django.utils.timezone import localtime
from django.utils.translation import gettext_lazy as _

import xlsxwriter
from ipware import get_client_ip as ipware_get_client_ip
from markdown import markdown as markdown_function

from daiquiri.core.constants import ACCESS_LEVEL_INTERNAL, ACCESS_LEVEL_PRIVATE, ACCESS_LEVEL_PUBLIC, GROUPS

if sys.version_info.major >= 3:
    long_type = int
else:
    long_type = long  # noqa: F821


def import_class(string):
    module_name, class_name = string.rsplit('.', 1)
    return getattr(importlib.import_module(module_name), class_name)


def get_script_alias(request):
    return request.path[:-len(request.path_info)]


def get_referer(request, default=None):
    return request.META.get('HTTP_REFERER', default)


def get_client_ip(request):
    client_ip, is_routable = ipware_get_client_ip(request)

    if client_ip:
        try:
            interface = ipaddress.IPv6Interface('%s/%i' % (client_ip, settings.IPV6_PRIVACY_MASK))
        except ipaddress.AddressValueError:
            interface = ipaddress.IPv4Interface('%s/%i' % (client_ip, settings.IPV4_PRIVACY_MASK))

        return str(interface.network.network_address)
    else:
        return None


def get_referer_path_info(request, default=None):
    referer = request.META.get('HTTP_REFERER', None)
    if not referer:
        return default

    script_alias = get_script_alias(request)
    return urlparse(referer).path[len(script_alias):]


def get_next(request):
    next = request.POST.get('next')
    current = request.path_info

    if next in (current, None):
        return get_script_alias(request) + '/'
    else:
        return get_script_alias(request) + next


def get_model_field_meta(*models):
    meta = defaultdict(lambda: defaultdict(dict))

    for model in models:
        for field in model._meta.get_fields():
            if hasattr(field, 'verbose_name'):
                meta[model._meta.model_name][field.name]['verbose_name'] = field.verbose_name
            if hasattr(field, 'help_text'):
                meta[model._meta.model_name][field.name]['help_text'] = field.help_text

    return meta


def get_detail_fields(detail_keys):
    fields = []

    # add a field for each detail key
    for detail_key in detail_keys:

        if 'options' in detail_key:
            choices = [(option['id'], option['label']) for option in detail_key['options']]
        else:
            choices = []

        if detail_key['data_type'] == 'text':
            field = forms.CharField(widget=forms.TextInput(attrs={'placeholder': detail_key['label']}))
        elif detail_key['data_type'] == 'textarea':
            field = forms.CharField(widget=forms.Textarea(attrs={'placeholder': detail_key['label']}))
        elif detail_key['data_type'] == 'select':
            field = forms.ChoiceField(choices=choices)
        elif detail_key['data_type'] == 'radio':
            field = forms.ChoiceField(choices=choices, widget=forms.RadioSelect)
        elif detail_key['data_type'] == 'multiselect':
            field = forms.MultipleChoiceField(choices=choices)
        elif detail_key['data_type'] == 'checkbox':
            field = forms.MultipleChoiceField(choices=choices, widget=forms.CheckboxSelectMultiple)
        else:
            raise Exception('Unknown detail key data type.')

        if 'label' in detail_key:
            field.label = detail_key['label']

        if 'required' in detail_key:
            field.required = detail_key['required']

        if 'help_text' in detail_key:
            field.help_text = detail_key['help_text']

        fields.append((detail_key['key'], field))

    return fields


def get_admin_emails():
    return [user.email for user in User.objects.filter(is_superuser=True)]


def get_permission_emails(permissions):
    permissions_queryset = Permission.objects
    for permission in permissions:
        app_label, codename = permission.split('.')
        permissions_queryset = permissions_queryset.filter(content_type__app_label=app_label, codename=codename)

    users = User.objects.filter(
        Q(groups__permissions__in=permissions_queryset) |
        Q(user_permissions__in=permissions_queryset)
    ).distinct()

    return [user.email for user in users]


def get_doi_url(doi):
    return 'https://doi.org/{}'.format(doi.rstrip('/')) if doi else None


def get_doi(doi_url):
    return urlparse(doi_url).path.lstrip('/') if doi_url else None


def filter_by_access_level(user, items):
    filtered_items = []

    for item in items:
        # check for the access level of the queue
        if 'access_level' not in item or item['access_level'] == ACCESS_LEVEL_PUBLIC:
            filtered_items.append(item)
        elif item['access_level'] == ACCESS_LEVEL_INTERNAL:
            if user is not None:
                if user.is_authenticated:
                    filtered_items.append(item)
        elif item['access_level'] == ACCESS_LEVEL_PRIVATE and 'groups' in item:
            if user is not None:
                if user.is_authenticated and user.groups.filter(name__in=item['groups']).exists():
                    filtered_items.append(item)

    return filtered_items


def human2bytes(string):
    if not string:
        return 0

    m = re.match(r'([0-9.]+)\s*([A-Za-z]+)', string)
    number, unit = float(m.group(1)), m.group(2).strip().lower()

    if unit == 'kb' or unit == 'k':
        return number * 1000
    elif unit == 'mb' or unit == 'm':
        return number * 1000**2
    elif unit == 'gb' or unit == 'g':
        return number * 1000**3
    elif unit == 'tb' or unit == 't':
        return number * 1000**4
    elif unit == 'pb' or unit == 'p':
        return number * 1000**5
    elif unit == 'kib':
        return number * 1024
    elif unit == 'mib':
        return number * 1024**2
    elif unit == 'gib':
        return number * 1024**3
    elif unit == 'tib':
        return number * 1024**4
    elif unit == 'pib':
        return number * 1024**5


def markdown(md):
    return markdown_function(md, extensions=[
                                    'fenced_code',
                                    'attr_list',
                                    'codehilite'
                                    ],
                                 extension_configs={
                                    'codehilite':{
                                        'guess_lang':'false'
                                    }
                                 }
                             )


def make_query_dict_upper_case(input_dict):
    output_dict = input_dict.copy()

    for key in input_dict.keys():
        if key.upper() != key:
            values = output_dict.getlist(key)

            if key.upper() in output_dict:
                output_dict.appendlist(key.upper(), values)
            else:
                output_dict.setlist(key.upper(), values)

            output_dict.pop(key)

    return output_dict


def fix_for_json(elements):
    # first convert to a list
    elements_list = list(elements)

    # then, fix stuff or run the function recursively
    for i in range(len(elements_list)):
        # for a list run fix_for_json recursively
        if isinstance(elements_list[i], (list, tuple)):
            elements_list[i] = fix_for_json(elements_list[i])

        # check the float fields for nan
        elif isinstance(elements_list[i], float) and math.isnan(elements_list[i]):
            elements_list[i] = None

        # convert a long fields to a strings
        elif isinstance(elements_list[i], long_type) and elements_list[i] > 9007199254740991:
            elements_list[i] = str(elements_list[i])

    return elements_list


def setup_group(name):
    group, created = Group.objects.get_or_create(name=name)
    group.permissions.clear()

    for row in GROUPS[name]:
        app_label, codename = row.split('.')
        try:
            permission = Permission.objects.get(content_type__app_label=app_label, codename=codename)
            group.permissions.add(permission)
        except Permission.DoesNotExist:
            pass

    return group, created


def send_mail(request, template_prefix, context, to_emails, cc_emails=[], bcc_emails=[]):
    '''
    This is heavily inspired by allauth.account.adapter.
    https://github.com/pennersr/django-allauth/blob/master/allauth/account/adapter.py
    '''
    # get current site
    site = get_current_site(request)

    # add site to context
    context['current_site'] = site

    # render subject from template and remove superfluous line breaks
    subject = render_to_string(f'{template_prefix}_subject.txt', context)
    subject = " ".join(subject.splitlines()).strip()

    # add site name to subject
    site = get_current_site(request)
    subject = f"[{site.name}] {subject}"

    # get from email
    from_email = settings.DEFAULT_FROM_EMAIL

    # get reply_to
    reply_to = settings.EMAIL_REPLY_TO

    # render bodie(s)
    bodies = {}
    for ext in ['html', 'txt']:
        try:
            template_name = f'{template_prefix}_message.{ext}'
            bodies[ext] = render_to_string(template_name,
                                           context).strip()
        except TemplateDoesNotExist:
            if ext == 'txt' and not bodies:
                # We need at least one body
                raise
    if 'txt' in bodies:
        msg = EmailMultiAlternatives(subject, bodies['txt'], from_email, to_emails,
                                     cc=cc_emails, bcc=bcc_emails, reply_to=reply_to)
        if 'html' in bodies:
            msg.attach_alternative(bodies['html'], 'text/html')
    else:
        msg = EmailMessage(subject, bodies['html'], from_email, to_emails,
                           cc=cc_emails, bcc=bcc_emails, reply_to=reply_to)
        msg.content_subtype = 'html'  # Main content is now text/html

    msg.send()


def render_to_csv(request, filename, columns, rows):
    response = HttpResponse(content_type='text/csv', charset='utf-8')
    response['Content-Disposition'] = (f'attachment; filename="{filename}.csv"').encode()

    writer = csv.writer(response)

    writer.writerow(tuple(columns))

    for row in rows:
        writer.writerow(tuple(row))

    return response


def render_to_xlsx(request, filename, columns, rows):
    response = HttpResponse(content_type='application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
                            charset='utf-8')
    response['Content-Disposition'] = (f'attachment; filename="{filename}.xlsx"').encode()

    workbook = xlsxwriter.Workbook(response, {
        'in_memory': True,
        'strings_to_formulas': False
    })
    worksheet = workbook.add_worksheet()
    bold = workbook.add_format({'bold': True})

    for j, column in enumerate(columns):
        worksheet.write(0, j, str(column), bold)

    for i, row in enumerate(rows):
        for j, cell in enumerate(row):
            if isinstance(cell, list):
                worksheet.write(i + 1, j, ', '.join(cell))
            elif isinstance(cell, bool):
                worksheet.write(i + 1, j, str(_('yes') if cell else _('no')))
            elif isinstance(cell, datetime):
                worksheet.write(i + 1, j, localtime(cell).strftime("%Y-%m-%d %H:%M:%S"))
            else:
                try:
                    worksheet.write(i + 1, j, unicode(cell))
                except NameError:
                    worksheet.write(i + 1, j, str(cell))

    workbook.close()

    return response


def render_to_xml(request, renderer, data, filename=None, content_type='application/xml'):
    # crete xml structure
    xml = renderer.render(data)

    # prettify xml
    pretty_xml = minidom.parseString(xml).toprettyxml()

    response = HttpResponse(pretty_xml, content_type=content_type)
    if filename:
        response['Content-Disposition'] = f'filename="{filename}"'
    return response


def handle_file_upload(directory, file):
    file_path = os.path.join(directory, file.name)

    os.makedirs(directory, exist_ok=True)

    with open(file_path, 'wb+') as destination:
        for chunk in file.chunks():
            destination.write(chunk)

    return file_path


def sanitize_str(strval):
    return re.sub(r'[^a-zA-Z0-9]', '_', strval.lower())


def get_file_size(file_path):
    try:
        return os.stat(file_path).st_size
    except (FileNotFoundError, TypeError):
        return 0


def get_date_display(value):
    return date(value, settings.DATETIME_FORMAT)



from django.contrib.auth.models import Group, User

from rest_framework import serializers

from allauth.account.models import EmailAddress

from daiquiri.core.serializers import JSONDictField
from daiquiri.core.utils import get_date_display

from .models import Profile


class UserSerializer(serializers.ModelSerializer):

    date_joined_label = serializers.SerializerMethodField(read_only=True)

    class Meta:
        model = User
        fields = (
            'last_login',
            'is_superuser',
            'username',
            'first_name',
            'last_name',
            'email',
            'is_staff',
            'is_active',
            'date_joined',
            'date_joined_label',
            'groups'
        )
        read_only_fields = (
            'last_login',
            'is_superuser',
            'username',
            'is_staff',
            'date_joined'
        )

    def get_date_joined_label(self, obj):
        return get_date_display(obj.date_joined)


class GroupSerializer(serializers.ModelSerializer):

    class Meta:
        model = Group
        fields = ('id', 'name')


class EmailAddressSerializer(serializers.ModelSerializer):

    class Meta:
        model = EmailAddress
        fields = ('email', 'verified')


class ProfileSerializer(serializers.ModelSerializer):

    user = UserSerializer()
    emails = serializers.SerializerMethodField(read_only=True)
    details = JSONDictField(allow_null=True)
    attributes = JSONDictField(allow_null=True)

    class Meta:
        model = Profile
        fields = (
            'id',
            'full_name',
            'user',
            'is_confirmed',
            'is_pending',
            'emails',
            'details',
            'attributes',
            'user_admin_url',
            'profile_admin_url'
        )

    def update(self, obj, validated_data):
        if 'user' in validated_data:
            user = validated_data.pop('user')

            # update the user for this profile separately
            obj.user.first_name = user['first_name']
            obj.user.last_name = user['last_name']
            obj.user.groups.set(user['groups'])
            obj.user.save()

        return super().update(obj, validated_data)

    def get_emails(self, obj):
        emails = EmailAddress.objects.filter(user=obj.user)
        serializer = EmailAddressSerializer(instance=emails, many=True)
        return serializer.data



import logging

from django.contrib.auth.models import User
from django.db import models
from django.urls import reverse
from django.utils.translation import gettext_lazy as _

from .signals import user_activated, user_confirmed, user_disabled, user_enabled, user_rejected
from .utils import get_full_name

logger = logging.getLogger(__name__)


class Profile(models.Model):

    user = models.OneToOneField(User, on_delete=models.CASCADE)

    is_pending = models.BooleanField(
        default=False,
        help_text='Designates whether the user waiting on confirmation by a manager.')
    is_confirmed = models.BooleanField(
        default=False,
        help_text='Designates whether the user was confirmed by a manager.')
    details = models.JSONField(null=True, blank=True)
    attributes = models.JSONField(null=True, blank=True)
    consent = models.BooleanField(
        default=False,
        help_text='Designates whether the user has agreed to the terms of use.',
        verbose_name='Consent'
    )

    class Meta:
        ordering = ('user__last_name', 'user__last_name', 'user__username')

        verbose_name = _('Profile')
        verbose_name_plural = _('Profiles')

    def __str__(self):
        return self.user.username

    @property
    def full_name(self):
        return get_full_name(self.user)

    def user_admin_url(self):
        return reverse('admin:auth_user_change', args=[self.user.id])

    def profile_admin_url(self):
        return reverse('admin:daiquiri_auth_profile_change', args=[self.id])

    def confirm(self, request):
        self.is_confirmed = True
        self.save()

        user_confirmed.send(sender=self.__class__, request=request, user=self.user)

    def reject(self, request):
        self.is_pending = False
        self.user.is_active = False
        self.user.save(update_fields=['is_active'])
        self.save()

        user_rejected.send(sender=self.__class__, request=request, user=self.user)

    def activate(self, request):
        self.is_confirmed = True
        self.is_pending = False
        self.save()

        user_activated.send(sender=self.__class__, request=request, user=self.user)

    def disable(self, request):
        self.user.is_active = False
        self.user.save(update_fields=['is_active'])

        user_disabled.send(sender=self.__class__, request=request, user=self.user)

    def enable(self, request):
        self.user.is_active = True
        self.user.save(update_fields=['is_active'])

        user_enabled.send(sender=self.__class__, request=request, user=self.user)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-07-05 13:02
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0007_alter_validators_add_error_messages'),
        ('daiquiri_metadata', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='column',
            name='published_for',
            field=models.ManyToManyField(blank=True, to='auth.Group'),
        ),
        migrations.AddField(
            model_name='database',
            name='published_for',
            field=models.ManyToManyField(blank=True, to='auth.Group'),
        ),
        migrations.AddField(
            model_name='function',
            name='published_for',
            field=models.ManyToManyField(blank=True, to='auth.Group'),
        ),
        migrations.AddField(
            model_name='table',
            name='published_for',
            field=models.ManyToManyField(blank=True, to='auth.Group'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-04-13 10:25
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_contact', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='contactmessage',
            options={'ordering': ('status', 'created', 'author'), 'verbose_name': 'Contact message', 'verbose_name_plural': 'Contact messages'},
        ),
        migrations.AlterField(
            model_name='contactmessage',
            name='status',
            field=models.CharField(choices=[('ACTIVE', 'active'), ('CLOSED', 'closed'), ('SPAM', 'spam')], max_length=8),
        ),
    ]



from datetime import datetime

from django.core.management.base import BaseCommand

from daiquiri.oai.adapter import OaiAdapter
from daiquiri.oai.models import Record
from daiquiri.oai.utils import update_records


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument('--resource_type', default=None, help='Update only a specific resource type.')
        parser.add_argument('--delete', action='store_true', help='Delete all records first.')

    def handle(self, *args, **options):

        adapter = OaiAdapter()

        if options['delete']:
            # delete and bulk ingest all records
            Record.objects.all().delete()

            records = []
            for resource_type, resource in adapter.get_resource_list():
                if options['resource_type'] in [None, resource_type]:
                    resource_id, identifier, datestamp, set_spec, public = adapter.get_record(resource_type, resource)

                    if public is True:
                        for metadata_prefix in adapter.get_metadata_prefixes(resource_type):
                            records.append(Record(
                                identifier=identifier,
                                metadata_prefix=metadata_prefix,
                                datestamp=datestamp,
                                set_spec=set_spec,
                                deleted=False,
                                resource_type=resource_type,
                                resource_id=resource_id
                            ))

            Record.objects.bulk_create(records)
        else:
            # mark deleted or unpublished records as deleted
            for record in Record.objects.all():
                resource = adapter.get_resource(record)
                if resource is None:
                    record.datestamp = datetime.today()
                    record.deleted = True
                    record.save()

            # update records one by one
            for resource_type, resource in adapter.get_resource_list():
                if options['resource_type'] in [None, resource_type]:
                    update_records(resource_type, resource)



from django import forms
from django.conf import settings
from django.contrib import admin

from .models import Column, Function, Schema, Table


class SchemaAdminForm(forms.ModelForm):
    license = forms.ChoiceField(choices=settings.LICENSE_CHOICES, required=False)


class TableAdminForm(forms.ModelForm):
    license = forms.ChoiceField(choices=settings.LICENSE_CHOICES, required=False)


class SchemaAdmin(admin.ModelAdmin):
    form = SchemaAdminForm

    search_fields = ('name',)
    list_display = ('order', '__str__', 'access_level', 'metadata_access_level')
    list_display_links = ('__str__', )


class TableAdmin(admin.ModelAdmin):
    form = TableAdminForm

    search_fields = ('name', 'schema__name')
    list_display = ('order', '__str__', 'access_level', 'metadata_access_level')
    list_display_links = ('__str__', )


class ColumnAdmin(admin.ModelAdmin):
    search_fields = ('name', 'table__name', 'table__schema__name', 'datatype')
    list_display = ('order', '__str__', 'datatype', 'access_level', 'metadata_access_level')
    list_display_links = ('__str__', )

    # only show access_level, metadata_access_level, and groups when
    # settings.METADATA_COLUMN_PERMISSIONS is set
    if not settings.METADATA_COLUMN_PERMISSIONS:
        fields = (
            'table',
            'order',
            'name',
            'description',
            'unit',
            'ucd',
            'utype',
            'datatype',
            'arraysize',
            'principal',
            'indexed',
            'std'
        )


class FunctionAdmin(admin.ModelAdmin):
    search_fields = ('name', 'query_string')
    list_display = ('order', 'name', 'query_string')
    list_display_links = ('name', )


admin.site.register(Schema, SchemaAdmin)
admin.site.register(Table, TableAdmin)
admin.site.register(Column, ColumnAdmin)
admin.site.register(Function, FunctionAdmin)



from django.core.management.base import BaseCommand, CommandError
from django.utils.translation import gettext_lazy as _

from daiquiri.core.constants import ACCESS_LEVEL_CHOICES
from daiquiri.metadata.models import Schema


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument('schema', help='the schema to be updated')
        parser.add_argument('access_level', help='new access_level and metadata_access_level')

    def handle(self, *args, **options):

        if options['access_level'] not in dict(ACCESS_LEVEL_CHOICES):
            raise CommandError(_('Unknown access_level.'))

        schema = Schema.objects.get(name=options['schema'])
        schema.access_level = options['access_level']
        schema.metadata_access_level = options['access_level']
        schema.save()

        for table in schema.tables.all():
            table.access_level = options['access_level']
            table.metadata_access_level = options['access_level']
            table.save()



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-04-01 17:17
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_jobs', '0004_phases_to_char'),
    ]

    operations = [
        migrations.AlterField(
            model_name='job',
            name='owner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
    ]



from django.urls import include, path

from rest_framework import routers

from .views import UsersView
from .viewsets import GroupViewSet, ProfileViewSet, SettingsViewSet

app_name = 'auth'

router = routers.DefaultRouter()
router.register(r'profiles', ProfileViewSet)
router.register(r'settings', SettingsViewSet, basename='settings')
router.register(r'groups', GroupViewSet)

urlpatterns = [
    # user management
    path('users/', UsersView.as_view(), name='users'),

    # rest api
    path('api/', include(router.urls)),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-04-13 14:31
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_query', '0008_queryjob_pid'),
    ]

    operations = [
        migrations.AlterField(
            model_name='queryjob',
            name='query_language',
            field=models.CharField(max_length=8),
        ),
        migrations.AlterField(
            model_name='queryjob',
            name='queue',
            field=models.CharField(max_length=16),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-03-10 16:15
from __future__ import unicode_literals

from django.db import migrations
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_auth', '0001_profile_model'),
    ]

    operations = [
        migrations.AddField(
            model_name='profile',
            name='attributes',
            field=jsonfield.fields.JSONField(blank=True, null=True),
        ),
    ]



from django.urls import reverse

from daiquiri.core.utils import get_admin_emails, get_permission_emails, send_mail


def get_manager_emails():
    return get_permission_emails((
        'daiquiri_contact.view_contactmessage',
    )) + get_admin_emails()


def send_contact_message(request, message):
    # sends an email to the admins, managers and the user once a message was submitted.
    context = {
        'user': message.user,
        'author': message.author,
        'email': message.email,
        'subject': message.subject,
        'message': message.message,
        'url': request.build_absolute_uri(reverse('contact:messages'))
    }
    send_mail(request, 'contact/email/new_message_admin', context, get_manager_emails())
    send_mail(request, 'contact/email/new_message_user', context, [message.email])



from django.conf import settings

from rest_framework import serializers

from ..models import Column, Function, Schema, Table


class ColumnSerializer(serializers.ModelSerializer):

    class Meta:
        model = Column
        fields = (
            'id',
            'order',
            'name',
            'query_strings',
            'description',
            'unit',
            'ucd',
            'utype',
            'datatype',
            'arraysize',
            'principal',
            'indexed',
            'std'
        )


class TableSerializer(serializers.ModelSerializer):

    columns = serializers.SerializerMethodField()

    class Meta:
        model = Table
        fields = (
            'id',
            'order',
            'name',
            'query_strings',
            'description',
            'type',
            'utype',
            'columns'
        )

    def get_columns(self, obj):
        # filter the columns which are published for the groups of the user
        if not settings.METADATA_COLUMN_PERMISSIONS:
            queryset = obj.columns.all()
        else:
            queryset = obj.columns.filter_by_access_level(self.context['request'].user)
        return ColumnSerializer(queryset, context=self.context, many=True).data


class SchemaSerializer(serializers.ModelSerializer):

    tables = serializers.SerializerMethodField()

    class Meta:
        model = Schema
        fields = (
            'id',
            'order',
            'name',
            'query_strings',
            'description',
            'utype',
            'tables'
        )

    def get_tables(self, obj):
        # filter the tables which are published for the groups of the user
        queryset = obj.tables.filter_by_access_level(self.context['request'].user)
        return TableSerializer(queryset, context=self.context, many=True).data


class FunctionSerializer(serializers.ModelSerializer):

    class Meta:
        model = Function
        fields = (
            'id',
            'order',
            'name',
            'description',
            'query_string'
        )



from django.core.exceptions import ValidationError

from rest_framework import filters

from daiquiri.auth.validators import DaiquiriUsernameValidator


class SpamBackend(filters.BaseFilterBackend):


    def filter_queryset(self, request, queryset, view):

        from daiquiri.contact.models import ContactMessage

        spam = request.GET.get('spam')

        if spam is not None:
            if spam == 'true':
                queryset = queryset.filter(status=ContactMessage.STATUS_SPAM)
            else:
                queryset = queryset.exclude(status=ContactMessage.STATUS_SPAM)

        return queryset



class DefaultMessageFilter:

    CHOICES = (
        ("no_filter", "Show to all users"),
        ("logged_in_users", "Show to the logged in users only"),
        ("user_has_not_consented", "Show to user who has not consented yet"),
        ("user_has_invalid_username", "Show to user with an invalid username"),
    )

    def no_filter(request):
        return True

    def logged_in_users(request):
        if request.user.is_authenticated:
            return True
        return False

    def user_has_not_consented(request):
        if request.user.is_authenticated:
            if not request.user.profile.consent:
                return True
        return False

    def user_has_invalid_username(request):
        if request.user.is_authenticated:
            try:
                DaiquiriUsernameValidator()(request.user.username)
            except ValidationError:
                return True
            return False



from datetime import datetime

from . import XMLRenderer
from .vosi import CapabilitiesRendererMixin, TablesetRendererMixin


class VoresourceRendererMixin(CapabilitiesRendererMixin, TablesetRendererMixin):

    def render_voresource(self, metadata):
        self.start('ri:Resource', {
            'created': self.render_date(metadata.get('created')),
            'updated': self.render_date(metadata.get('updated')),
            'status': metadata.get('status'),
            'xsi:type': metadata.get('type'),
            'xmlns:xsi': 'http://www.w3.org/2001/XMLSchema-instance',
            'xmlns:ri': 'http://www.ivoa.net/xml/RegistryInterface/v1.0',
            'xmlns:vg': 'http://www.ivoa.net/xml/VORegistry/v1.0',
            'xmlns:vr': 'http://www.ivoa.net/xml/VOResource/v1.0',
            'xmlns:vs': 'http://www.ivoa.net/xml/VODataService/v1.1',
            'xmlns:cs': 'http://www.ivoa.net/xml/ConeSearch/v1.0',
            'xsi:schemaLocation': 'http://www.ivoa.net/xml/RegistryInterface/v1.0 http://www.ivoa.net/xml/VORegistry/v1.0 http://www.ivoa.net/xml/VOResource/v1.0 http://www.ivoa.net/xml/VODataService/v1.1'  # noqa: E501
        })

        self.node('title', {}, metadata.get('title'))
        self.node('identifier', {}, metadata.get('identifier'))

        if metadata.get('short_name'):
            self.node('shortName', {}, metadata.get('short_name'))

        self.render_curation(metadata.get('curation', {}))
        self.render_content(metadata.get('content', {}))

        rights = metadata.get('rights')
        if rights:
            self.node('rights', {}, metadata.get('rights'))

        for capability in metadata.get('capabilities', []):
            self.render_capability(capability)

        full = metadata.get('full')
        if full:
            self.node('full', {}, full)

        managed_authority = metadata.get('managed_authority')
        if managed_authority:
            self.node('managedAuthority', {}, managed_authority)

        managing_org = metadata.get('managing_org')
        if managing_org:
            self.node('managingOrg', {}, managing_org)

        tableset = metadata.get('tableset', [])
        if tableset:
            self.start('tableset')
            self.render_tableset(tableset, strict=True)
            self.end('tableset')

        self.end('ri:Resource')

    def render_curation(self, curation_metadata):
        self.start('curation')
        self.node('publisher', {}, curation_metadata.get('publisher'))

        creator = curation_metadata.get('creator')
        if creator:
            self.start('creator')
            self.node('name', {}, creator.get('name'))
            self.node('logo', {}, creator.get('logo'))
            self.end('creator')

        self.node('date', {'role': 'updated'}, self.render_date(curation_metadata.get('date')))

        contact = curation_metadata.get('contact')
        if contact:
            self.start('contact')
            self.node('name', {}, contact.get('name'))
            self.node('address', {}, contact.get('address'))
            self.node('email', {}, contact.get('email'))
            self.node('telephone', {}, contact.get('telephone'))
            self.end('contact')

        self.end('curation')

    def render_content(self, content_metadata):
        self.start('content')
        for subject in content_metadata.get('subjects', []):
            self.node('subject', {}, subject)
        self.node('description', {}, content_metadata.get('description'))
        self.node('referenceURL', {}, content_metadata.get('referenceURL'))
        self.node('type', {}, content_metadata.get('type'))
        self.end('content')

    def render_date(self, date):
        return datetime.strptime(date, '%Y-%m-%d').strftime('%Y-%m-%dT%H:%M:%SZ')


class VoresourceRenderer(VoresourceRendererMixin, XMLRenderer):

    def render_document(self, data, accepted_media_type=None, renderer_context=None):
        self.render_voresource(data)



from collections import OrderedDict

from rest_framework import viewsets
from rest_framework.exceptions import NotFound
from rest_framework.response import Response

from daiquiri.core.adapter import DatabaseAdapter
from daiquiri.core.utils import fix_for_json
from daiquiri.core.viewsets import RowViewSetMixin
from daiquiri.metadata.utils import get_user_columns

from .serializers import ColumnSerializer


class RowViewSet(RowViewSetMixin, viewsets.GenericViewSet):

    def list(self, request, *args, **kwargs):

        # get schema, table and column_names from the querystring
        schema_name = self.request.GET.get('schema')
        table_name = self.request.GET.get('table')
        column_names = self.request.GET.getlist('column')

        # get the columns which the user is allowed to access
        user_columns = get_user_columns(self.request.user, schema_name, table_name)

        if user_columns:
            # get the row query params from the request
            ordering, page, page_size, search, filters = self._get_query_params(user_columns)

            # filter by input column names by the the allowed columns
            if column_names:
                column_names = [column.name for column in user_columns if column.name in column_names]
            else:
                column_names = [column.name for column in user_columns]

            # get database adapter
            adapter = DatabaseAdapter()

            # query the database for the total number of rows
            count = adapter.count_rows(schema_name, table_name, column_names, search, filters)

            # query the paginated rowset
            results = adapter.fetch_rows(schema_name, table_name, column_names,
                                         ordering, page, page_size, search, filters)

            # return ordered dict to be send as json
            return Response(OrderedDict((
                ('count', count),
                ('results', fix_for_json(results)),
                ('next', self._get_next_url(page, page_size, count)),
                ('previous', self._get_previous_url(page))
            )))

        # if nothing worked, return 404
        raise NotFound()


class ColumnViewSet(viewsets.ViewSet):

    def list(self, request, *args, **kwargs):

        # get database, table and column_names from the querystring
        schema_name = self.request.GET.get('schema')
        table_name = self.request.GET.get('table')
        column_names = self.request.GET.getlist('column')

        # get the columns which the user is allowed to access
        user_columns = get_user_columns(self.request.user, schema_name, table_name)

        if user_columns:
            # filter by input column names by the the allowed columns
            if column_names:
                columns = [column for column in user_columns if column.name in column_names]
            else:
                columns = list(user_columns)

            return Response(ColumnSerializer(columns, many=True).data)

        # if nothing worked, return 404
        raise NotFound()



import logging
import re

from django.db import OperationalError, ProgrammingError

from .base import BaseDatabaseAdapter

logger = logging.getLogger(__name__)


class MySQLAdapter(BaseDatabaseAdapter):

    DATATYPES = {
        'char': {
            'datatype': 'char',
            'arraysize': True
        },
        'varchar': {
            'datatype': 'char',
            'arraysize': True
        },
        'text': {
            'datatype': 'char',
            'arraysize': True
        },
        'tinyint': {
            'datatype': 'boolean',
            'arraysize': False
        },
        'smallint': {
            'datatype': 'short',
            'arraysize': False
        },
        'int': {
            'datatype': 'int',
            'arraysize': False
        },
        'bigint': {
            'datatype': 'long',
            'arraysize': False
        },
        'float': {
            'datatype': 'float',
            'arraysize': False
        },
        'double': {
            'datatype': 'double',
            'arraysize': False
        },
        'timestamp': {
            'datatype': 'timestamp',
            'arraysize': False
        }
    }

    COLUMNTYPES = {
        'char': 'text',
        'boolean': 'boolean',
        'short': 'smallint',
        'int': 'int',
        'long': 'bigint',
        'float': 'float',
        'double': 'double'
    }

    search_stmt_template = '%s LIKE %%s'
    search_arg_template = '%%%s%%'

    def fetch_pid(self):
        return self.connection().connection.thread_id()

    def escape_identifier(self, identifier):
        # escape backticks within the identifier and backtick the string
        return '`{}`'.format(identifier.replace('`', '``'))

    def escape_string(self, string):
        return f"'{string}'"

    def build_query(self, schema_name, table_name, query, timeout, max_records):
        # construct the actual query
        params = {
            'schema': self.escape_identifier(schema_name),
            'table': self.escape_identifier(table_name),
            'query': query,
            'timeout': timeout,
            'max_records': max_records
        }

        if max_records is not None:
            return 'CREATE TABLE {schema}.{table} ENGINE=MyISAM ( {query} ) LIMIT {max_records};'.format(**params)
        else:
            return 'CREATE TABLE {schema}.{table} ENGINE=MyISAM ( {query} );'.format(**params)

    def build_sync_query(self, query, timeout, max_records):
        # construct the actual query
        params = {
            'query': query,
            'timeout': timeout,
            'max_records': max_records
        }

        if max_records is not None:
            return '{query} LIMIT {max_records};'.format(**params)
        else:
            return '{query};'.format(**params)

    def abort_query(self, pid):
        sql = 'KILL %(pid)i' % {'pid': pid}
        self.execute(sql)

    def fetch_size(self, schema_name, table_name):
        sql = 'SELECT data_length + index_length AS size FROM `information_schema`.`tables` WHERE `table_schema` = %s AND table_name = %s;'  # noqa: E501
        size = self.fetchone(sql, (schema_name, table_name))[0]

        # log values and return
        logger.debug('size = %d', size)
        return size

    def fetch_nrows(self, schema_name, table_name):
        sql = 'SELECT table_rows as nrows FROM `information_schema`.`tables` WHERE `table_schema` = %s AND table_name = %s;'  # noqa: E501
        nrows = self.fetchone(sql, (schema_name, table_name))[0]

        # log values and return
        logger.debug('size = %d', nrows)
        return nrows

    def fetch_tables(self, schema_name):
        # escape input
        escaped_schema_name = self.escape_identifier(schema_name)

        # prepare sql string
        sql = f'SHOW FULL TABLES FROM {escaped_schema_name}'

        # execute query
        try:
            rows = self.fetchall(sql)
        except OperationalError as e:
            logger.error('Could not fetch from %s (%s)', schema_name, e)
            return []
        else:
            return [{
                'name': row[0],
                'type': 'view' if row[1] == 'VIEW' else 'table'
            } for row in rows]

    def fetch_table(self, schema_name, table_name):
        # prepare sql string
        sql = f'SHOW FULL TABLES FROM {self.escape_identifier(schema_name)} LIKE {self.escape_string(table_name)}'

        # execute query
        try:
            row = self.fetchone(sql)
        except OperationalError as e:
            logger.error('Could not fetch %s.%s (%s)', schema_name, table_name, e)
            return {}
        else:
            if row is None:
                logger.info('Could not fetch %s.%s. Check if table and schema exist.', schema_name, table_name)
                return {}
            else:
                return {
                    'name': row[0],
                    'type': 'view' if row[1] == 'VIEW' else 'table'
                }

    def fetch_columns(self, schema_name, table_name):
        # prepare sql string
        sql = f'SHOW FULL COLUMNS FROM {self.escape_identifier(schema_name)}.{self.escape_identifier(table_name)};'

        # execute query
        try:
            rows = self.fetchall(sql)
        except ProgrammingError as e:
            logger.error('Could not fetch columns from %s.%s (%s)', schema_name, table_name, e)
            return []
        else:
            columns = []
            for row in rows:
                datatype, arraysize = self._convert_datatype(row[1])

                columns.append({
                    'name': row[0],
                    'datatype': datatype,
                    'arraysize': arraysize,
                    'indexed': bool(row[4])
                })

            return columns

    def fetch_column(self, schema_name, table_name, column_name):
        # prepare sql string
        sql = f'SHOW FULL COLUMNS FROM {self.escape_identifier(schema_name)}.{self.escape_identifier(table_name)} WHERE `Field` = {self.escape_string(column_name)}'  # noqa: E501

        # execute query
        try:
            row = self.fetchone(sql)
        except ProgrammingError as e:
            logger.error('Could not fetch %s.%s.%s (%s)', schema_name, table_name, column_name, e)
            return {}
        else:
            if row is None:
                logger.info(f'Could not fetch {schema_name}.{table_name}.{column_name}. Check if column, table and schema exist.')
                return {}
            else:
                return {
                    'name': row[0],
                    'datatype': row[1],
                    'indexed': bool(row[4])
                }

    def fetch_column_names(self, schema_name, table_name):
        # prepare sql string
        sql = f'SHOW COLUMNS FROM {self.escape_identifier(schema_name)}.{self.escape_identifier(table_name)}'
        return [column[0] for column in self.fetchall(sql)]

    def rename_table(self, schema_name, table_name, new_table_name):
        sql = 'RENAME TABLE {schema}.{table} to {schema}.{new_table};'.format(
            schema=self.escape_identifier(schema_name),
            table=self.escape_identifier(table_name),
            new_table=self.escape_identifier(new_table_name),
        )

        self.execute(sql)

    def _convert_datatype(self, datatype_string):
        result = re.match(r'([a-z]+)\(*(\d*)\)*', datatype_string)

        if result:
            native_datatype = result.group(1)

            try:
                native_arraysize = int(result.group(2))
            except ValueError:
                native_arraysize = None

            if native_datatype in self.DATATYPES:
                datatype = self.DATATYPES[native_datatype]['datatype']

                if self.DATATYPES[native_datatype]['arraysize']:
                    arraysize = native_arraysize
                else:
                    arraysize = None

                return datatype, arraysize
            else:
                return native_datatype, native_arraysize
        else:
            return datatype_string, None



# Generated by Django 2.1.3 on 2018-11-20 08:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_files', '0004_django2'),
    ]

    operations = [
        migrations.AddField(
            model_name='directory',
            name='layout',
            field=models.BooleanField(default=False, help_text='Use the page layout with the content.', verbose_name='Layout'),
            preserve_default=False,
        ),
    ]



from pathlib import Path
from unittest import mock

import pytest

from django.conf import settings
from django.test import override_settings
from django.urls import reverse

from ..models import QueryJob

users = (
    ('admin', 'admin'),
    ('user', 'user'),
    ('test', 'test'),
    ('evil', 'evil'),
    ('anonymous', None),
)

status_map = {
    'list': {
        'admin': 200, 'user': 200, 'test': 200, 'evil': 200, 'anonymous': 403
    },
    'detail': {
        'admin': 404, 'user': 200, 'test': 404, 'evil': 404, 'anonymous': 403
    },
    'create_internal': {
        'admin': 201, 'user': 201, 'test': 201, 'evil': 201, 'anonymous': 403
    },
    'create_private': {
        'admin': 400, 'user': 400, 'test': 201, 'evil': 400, 'anonymous': 403
    },
    'update': {
        'admin': 404, 'user': 200, 'test': 404, 'evil': 404, 'anonymous': 403
    },
    'delete': {
        'admin': 404, 'user': 204, 'test': 404, 'evil': 404, 'anonymous': 403
    }
}

urlnames = {
    'list': 'query:job-list',
    'detail': 'query:job-detail',
    'abort': 'query:job-abort',
    'create-download': 'query:job-create-download',
    'download': 'query:job-download',
    'stream': 'query:job-stream',
    'rows': 'query:job-rows',
    'columns': 'query:job-columns'
}

instances = [
    '679b5ce8-bbcf-41c2-ba45-2340910bcfae',
    '5527f2b0-7130-466d-b8d2-dbf6c5f700bf',
    'c87926e4-8e5f-4032-a764-826b6a39c171',
    '1b6c93ef-4161-4402-b1a7-d1237657e807'
]

public_queries = [
    'SELECT ra, dec, parallax, id FROM daiquiri_data_obs.stars'
]

internal_queries = [
    'SELECT ra, dec, parallax, id FROM daiquiri_data_obs.stars',
    'SELECT x, y, z, vx, vy, vz, id FROM daiquiri_data_sim.halos'
]

private_queries = [
    'SELECT id, "bool", "array", matrix FROM daiquiri_data_test.test'
]

format_keys = ('votable', 'csv', 'fits')


@pytest.mark.parametrize(('username', 'password'), users)
def test_list(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse(urlnames['list'])
    response = client.get(url)
    assert response.status_code == status_map['list'][username], response.json()

    if response.status_code == 200:
        if username == 'user':
            assert response.json()['count'] == 4
            assert response.json()['results'][0]['table_name'] == 'test_pending'
        else:
            assert response.json()['count'] == 0


@override_settings(QUERY_ANONYMOUS=True)
def test_list_anonymous(db, client):
    url = reverse(urlnames['list'])
    response = client.get(url)
    assert response.status_code == 200, response.json()

    if response.status_code == 200:
        assert response.json()['count'] == 0


@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_detail(db, client, username, password, pk):
    client.login(username=username, password=password)

    instance = QueryJob.objects.get(pk=pk)
    url = reverse(urlnames['detail'], args=[pk])
    response = client.get(url)
    assert response.status_code == status_map['detail'][username], response.json()

    if response.status_code == 200:
        assert response.json().get('table_name') == instance.table_name


@override_settings(QUERY_ANONYMOUS=True)
@pytest.mark.parametrize('query', public_queries)
def test_create_public(db, client, mocker, query):
    mocker.patch(settings.ADAPTER_DATABASE + '.submit_query', mock.Mock())
    mocker.patch(settings.ADAPTER_DATABASE + '.fetch_columns', mock.Mock(return_value=[]))
    mocker.patch(settings.ADAPTER_DATABASE + '.fetch_size', mock.Mock(return_value=100))
    mocker.patch(settings.ADAPTER_DATABASE + '.count_rows', mock.Mock(return_value=100))
    mocker.patch(settings.ADAPTER_DATABASE + '.create_user_schema_if_not_exists', mock.Mock())

    url = reverse(urlnames['list'])
    response = client.post(url, {
        'query_language': 'adql-2.0',
        'query': query
    })
    assert response.status_code == 201, response.json()


@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('query', internal_queries)
def test_create_internal(db, client, mocker, username, password, query):
    mocker.patch(settings.ADAPTER_DATABASE + '.submit_query', mock.Mock())
    mocker.patch(settings.ADAPTER_DATABASE + '.fetch_columns', mock.Mock(return_value=[]))
    mocker.patch(settings.ADAPTER_DATABASE + '.fetch_size', mock.Mock(return_value=100))
    mocker.patch(settings.ADAPTER_DATABASE + '.count_rows', mock.Mock(return_value=100))
    mocker.patch(settings.ADAPTER_DATABASE + '.create_user_schema_if_not_exists', mock.Mock())

    client.login(username=username, password=password)

    url = reverse(urlnames['list'])
    response = client.post(url, {
        'query_language': 'adql-2.0',
        'query': query
    })
    assert response.status_code == status_map['create_internal'][username], response.json()


@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('query', private_queries)
def test_create_private(db, client, mocker, username, password, query):
    mocker.patch(settings.ADAPTER_DATABASE + '.submit_query', mock.Mock())
    mocker.patch(settings.ADAPTER_DATABASE + '.fetch_columns', mock.Mock(return_value=[]))
    mocker.patch(settings.ADAPTER_DATABASE + '.fetch_size', mock.Mock(return_value=100))
    mocker.patch(settings.ADAPTER_DATABASE + '.count_rows', mock.Mock(return_value=100))
    mocker.patch(settings.ADAPTER_DATABASE + '.create_user_schema_if_not_exists', mock.Mock())

    client.login(username=username, password=password)

    url = reverse(urlnames['list'])
    response = client.post(url, {
        'query_language': 'adql-2.0',
        'query': query
    })
    assert response.status_code == status_map['create_private'][username], response.json()


@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_update(db, client, mocker, username, password, pk):
    mocker.patch(settings.ADAPTER_DATABASE + '.rename_table', mock.Mock())

    client.login(username=username, password=password)

    instance = QueryJob.objects.get(pk=pk)
    instance_table_name = instance.table_name
    url = reverse(urlnames['detail'], args=[pk])
    response = client.put(url, {
        'table_name': 'renamed',
    }, content_type='application/json')
    assert response.status_code == status_map['update'][username], response.json()

    instance.refresh_from_db()
    assert instance.table_name == ('renamed' if response.status_code == 200 else instance_table_name)


@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_delete(db, client, mocker, username, password, pk):
    mocker.patch(settings.ADAPTER_DATABASE + '.drop_table', mock.Mock())

    client.login(username=username, password=password)

    instance = QueryJob.objects.get(pk=pk)
    instance_phase = instance.phase
    url = reverse(urlnames['detail'], args=[pk])
    response = client.delete(url)
    assert response.status_code == status_map['delete'][username], response.json()

    instance.refresh_from_db()
    if response.status_code == 204:
        assert instance.phase == 'ARCHIVED' if instance_phase == 'COMPLETED' else instance_phase
    else:
        assert instance.phase == instance_phase


@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_abort(db, client, username, password, pk):
    client.login(username=username, password=password)

    instance = QueryJob.objects.get(pk=pk)
    instance_phase = instance.phase
    url = reverse(urlnames['abort'], args=[pk])
    response = client.put(url)
    assert response.status_code == status_map['update'][username], response.json()

    instance.refresh_from_db()
    if response.status_code == 200:
        assert instance.phase == 'ABORTED' if instance_phase == 'EXECUTING' else instance_phase
    else:
        assert instance.phase == instance_phase


@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_rows(db, client, username, password, pk):
    client.login(username=username, password=password)

    instance = QueryJob.objects.get(pk=pk)
    url = reverse(urlnames['rows'], args=[pk])
    response = client.get(url)

    if username == 'user':
        if instance.phase == 'COMPLETED':
            assert response.status_code == 200
            assert response.json()['count'] == 10000
        else:
            assert response.status_code == 400
    elif username == 'anonymous':
        assert response.status_code == 403
    else:
        assert response.status_code == 404


@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_columns(db, client, username, password, pk):
    client.login(username=username, password=password)

    instance = QueryJob.objects.get(pk=pk)
    url = reverse(urlnames['columns'], args=[pk])
    response = client.get(url)

    if username == 'user':
        assert response.status_code == 200
        assert len(response.json()) == (4 if instance.phase == 'COMPLETED' else 0)
    elif username == 'anonymous':
        assert response.status_code == 403
    else:
        assert response.status_code == 404


@pytest.mark.parametrize('format_key', format_keys)
def test_create_download_completed(db, client, format_key):
    client.login(username='user', password='user')

    pk = '1b6c93ef-4161-4402-b1a7-d1237657e807'
    instance = QueryJob.objects.get(pk=pk)
    url = reverse(urlnames['create-download'], args=[pk, 'table'])
    file_name = get_download_file_name(instance, format_key)

    # remove existing file
    Path(settings.QUERY_DOWNLOAD_DIR).joinpath(file_name).unlink(missing_ok=True)

    # file is not existing yet
    response = client.post(url, {
        'format_key': format_key
    })
    assert response.status_code == 200, response.json()

    # file exists
    response = client.post(url, {
        'format_key': format_key
    })
    assert response.status_code == 200, response.json()


@pytest.mark.parametrize('format_key', format_keys)
def test_create_download_executing(db, client, format_key):
    client.login(username='user', password='user')

    pk = '679b5ce8-bbcf-41c2-ba45-2340910bcfae'
    instance = QueryJob.objects.get(pk=pk)
    url = reverse(urlnames['create-download'], args=[pk, 'table'])
    file_name = get_download_file_name(instance, format_key)

    # remove existing file
    Path(settings.QUERY_DOWNLOAD_DIR).joinpath(file_name).unlink(missing_ok=True)

    # file is not existing yet
    response = client.post(url, {
        'format_key': format_key
    })
    assert response.status_code == 400, response.json()


@pytest.mark.parametrize('format_key', format_keys)
def test_stream_completed(db, client, format_key):
    client.login(username='user', password='user')

    pk = '1b6c93ef-4161-4402-b1a7-d1237657e807'
    instance = QueryJob.objects.get(pk=pk)
    url = reverse(urlnames['stream'], args=[pk, format_key])
    file_name = get_download_file_name(instance, format_key)

    # remove existing file
    Path(settings.QUERY_DOWNLOAD_DIR).joinpath(file_name).unlink(missing_ok=True)

    # file is not existing yet
    response = client.get(url)
    assert response.status_code == 200, response.json()


@pytest.mark.parametrize('format_key', format_keys)
def test_stream_executing(db, client, format_key):
    client.login(username='user', password='user')

    pk = '679b5ce8-bbcf-41c2-ba45-2340910bcfae'
    instance = QueryJob.objects.get(pk=pk)
    url = reverse(urlnames['stream'], args=[pk, format_key])
    file_name = get_download_file_name(instance, format_key)

    # remove existing file
    Path(settings.QUERY_DOWNLOAD_DIR).joinpath(file_name).unlink(missing_ok=True)

    # file is not existing yet
    response = client.get(url)
    assert response.status_code == 400, response.json()


def get_download_file_name(instance, format_key):
    file_name = f'{instance.owner.username}/{instance.table_name}'

    if format_key == 'votable':
        return file_name + '.xml'
    elif format_key == 'csv':
        return file_name + '.csv'
    elif format_key == 'fits':
        return file_name + '.fits'



from urllib.parse import urlencode

import pytest

from django.urls import reverse

users = (
    ('admin', 'admin'),
    ('user', 'user'),
    ('test', 'test'),
    ('anonymous', None),
)


@pytest.mark.parametrize(('username', 'password'), users)
def test_public_row_list(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse('serve:row-list') + '?' + urlencode({
        'schema': 'daiquiri_data_obs',
        'table': 'stars'
    })
    response = client.get(url)
    assert response.status_code == 200


@pytest.mark.parametrize(('username', 'password'), users)
def test_internal_row_list(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse('serve:row-list') + '?' + urlencode({
        'schema': 'daiquiri_data_sim',
        'table': 'halos'
    })
    response = client.get(url)
    assert response.status_code == 404 if username == 'anonymous' else 200


@pytest.mark.parametrize(('username', 'password'), users)
def test_private_row_list(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse('serve:row-list') + '?' + urlencode({
        'schema': 'daiquiri_data_test',
        'table': 'test'
    })
    response = client.get(url)
    assert response.status_code == 200 if username == 'test' else 404


@pytest.mark.parametrize(('username', 'password'), users)
def test_non_existing_schema_row_list(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse('serve:row-list') + '?' + urlencode({
        'schema': 'non_existing',
        'table': 'stars'
    })
    response = client.get(url)
    assert response.status_code == 404


@pytest.mark.parametrize(('username', 'password'), users)
def test_non_existing_table_row_list(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse('serve:row-list') + '?' + urlencode({
        'schema': 'daiquiri_data_obs',
        'table': 'non_existing'
    })
    response = client.get(url)
    assert response.status_code == 404


@pytest.mark.parametrize(('username', 'password'), users)
def test_non_existing_user_table_row_list(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse('serve:row-list') + '?' + urlencode({
        'schema': 'daiquiri_user_user',
        'table': 'non_existing'
    })
    response = client.get(url)
    assert response.status_code == 404


@pytest.mark.parametrize(('username', 'password'), users)
def test_public_column_list(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse('serve:column-list') + '?' + urlencode({
        'schema': 'daiquiri_data_obs',
        'table': 'stars'
    })
    response = client.get(url)
    assert response.status_code == 200


@pytest.mark.parametrize(('username', 'password'), users)
def test_internal_column_list(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse('serve:column-list') + '?' + urlencode({
        'schema': 'daiquiri_data_sim',
        'table': 'halos'
    })
    response = client.get(url)
    assert response.status_code == 404 if username == 'anonymous' else 200


@pytest.mark.parametrize(('username', 'password'), users)
def test_private_column_list(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse('serve:column-list') + '?' + urlencode({
        'schema': 'daiquiri_data_test',
        'table': 'test'
    })
    response = client.get(url)
    assert response.status_code == 200 if username == 'test' else 404


@pytest.mark.parametrize(('username', 'password'), users)
def test_non_existing_schema_column_list(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse('serve:column-list') + '?' + urlencode({
        'schema': 'non_existing',
        'table': 'stars'
    })
    response = client.get(url)
    assert response.status_code == 404


@pytest.mark.parametrize(('username', 'password'), users)
def test_non_existing_table_column_list(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse('serve:column-list') + '?' + urlencode({
        'schema': 'daiquiri_data_obs',
        'table': 'non_existing'
    })
    response = client.get(url)
    assert response.status_code == 404


@pytest.mark.parametrize(('username', 'password'), users)
def test_non_existing_user_table_column_list(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse('serve:column-list') + '?' + urlencode({
        'schema': 'daiquiri_user_user',
        'table': 'non_existing'
    })
    response = client.get(url)
    assert response.status_code == 404



# -*- coding: utf-8 -*-
# Generated by Django 1.11.10 on 2018-02-16 17:14
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_metadata', '0015_change_doi_to_char'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='table',
            options={'ordering': ('schema__order', 'order'), 'permissions': (('view_table', 'Can view Table'),), 'verbose_name': 'Table', 'verbose_name_plural': 'Tables'},
        ),
        migrations.RenameField(
            model_name='table',
            old_name='database',
            new_name='schema',
        ),
        migrations.AlterField(
            model_name='database',
            name='attribution',
            field=models.TextField(blank=True, help_text='The desired attribution for the schema.', null=True, verbose_name='Attribution'),
        ),
        migrations.AlterField(
            model_name='database',
            name='description',
            field=models.TextField(blank=True, help_text='A brief description of the schema to be displayed in the user interface.', null=True, verbose_name='Description'),
        ),
        migrations.AlterField(
            model_name='database',
            name='groups',
            field=models.ManyToManyField(blank=True, help_text='The groups which have access to the schema.', to='auth.Group', verbose_name='Groups'),
        ),
        migrations.AlterField(
            model_name='database',
            name='long_description',
            field=models.TextField(blank=True, help_text='A more extensive description of the schema to be displayed on the public schema page.', null=True, verbose_name='Long description'),
        ),
        migrations.AlterField(
            model_name='database',
            name='name',
            field=models.CharField(help_text='Name of the schema on the database server.', max_length=256, verbose_name='Name'),
        ),
        migrations.AlterField(
            model_name='database',
            name='title',
            field=models.CharField(blank=True, help_text='Human readable title of the schema.', max_length=256, null=True, verbose_name='Title'),
        ),
    ]



from django import forms
from django.contrib import admin

from .models import AnnouncementMessage, ContactMessage, MessageFilter


class AnnouncementMessageAdminForm(forms.ModelForm):
    visibility_filter = forms.ChoiceField(choices=MessageFilter().CHOICES)


@admin.action(description="Make selected messages visible")
def make_visible(modeladmin, request, queryset):
    queryset.update(visible=True)


@admin.action(description="Make selected messages invisible")
def make_invisible(modeladmin, request, queryset):
    queryset.update(visible=False)


class ContactMessageAdmin(admin.ModelAdmin):
    search_fields = ("subject", "email", "author", "status", "user__username")
    list_display = ("subject", "email", "author", "status")


class AnnouncementMessageAdmin(admin.ModelAdmin):
    form = AnnouncementMessageAdminForm

    search_fields = ("title", "announcement")
    list_display = ("title", "visible", "announcement", "announcement_type", "updated")
    actions = [make_visible, make_invisible]

admin.site.register(ContactMessage, ContactMessageAdmin)
admin.site.register(AnnouncementMessage, AnnouncementMessageAdmin)



from django.conf import settings

from daiquiri.core.adapter.stream import BaseServiceAdapter
from daiquiri.core.utils import import_class


def CutOutAdapter():
    return import_class(settings.CUTOUT_ADAPTER)()


class BaseCutOutAdapter(BaseServiceAdapter):
    pass



import logging

from django.conf import settings
from django.contrib.auth.models import User
from django.db.models.signals import m2m_changed, post_delete, post_save
from django.dispatch import receiver

from allauth.account.signals import email_confirmed, password_changed, password_reset

from .models import Profile
from .signals import (
    user_activated,
    user_confirmed,
    user_created,
    user_deleted,
    user_disabled,
    user_enabled,
    user_groups_updated,
    user_rejected,
    user_updated,
)
from .utils import (
    send_activation,
    send_notify_activation,
    send_notify_confirmation,
    send_notify_password_changed,
    send_notify_rejection,
    send_request_activation,
    send_request_confirmation,
)

logger = logging.getLogger(__name__)


@receiver(post_save, sender=User)
def post_save_user(sender, **kwargs):
    if not kwargs.get('raw', False):
        user = kwargs['instance']

        if kwargs['created']:
            profile = Profile()
            profile.user = user
            profile.save()

            user_created.send(sender=User, user=user)
            logger.info('user \'%s\' created.', user.username)

        elif kwargs['update_fields'] is None:
            # a login triggers this handler with update_fields=last_login
            user_updated.send(sender=User, user=user)
            logger.info('user \'%s\' updated.', user.username)


@receiver(post_delete, sender=User)
def post_delete_user(sender, **kwargs):
    if not kwargs.get('raw', False):
        user = kwargs['instance']
        user_deleted.send(sender=User, user=user)
        logger.info('user \'%s\' deleted.', user.username)


@receiver(m2m_changed, sender=User.groups.through)
def m2m_changed_user(sender, **kwargs):
    if not kwargs.get('raw', False):
        user = kwargs['instance']

        # fire the signal only one per change
        if kwargs['action'] in ('post_add', 'post_remove', 'post_clear'):
            user_groups_updated.send(sender=User, user=user)
            logger.info('groups for user \'%s\' updated.', user.username)


@receiver(email_confirmed)
def email_confirmed_handler(sender, **kwargs):
    '''
    Gets notified when a user has validated his/her email address.
    '''
    request = kwargs['request']
    user = kwargs['email_address'].user

    logger.info('user \'%s\' validated his/her email address.', user.username)

    if settings.AUTH_WORKFLOW:
        user = kwargs['email_address'].user

        if user.profile.is_pending:
            if settings.AUTH_WORKFLOW == 'confirmation':
                send_request_confirmation(request, user)
            elif settings.AUTH_WORKFLOW == 'activation':
                send_request_activation(request, user)


@receiver(user_confirmed)
def user_confirmed_handler(sender, **kwargs):
    '''
    Gets notified when a user was confirmed by a manager.
    '''
    request = kwargs['request']
    user = kwargs['user']

    logger.info('user \'%s\' confirmed by \'%s\'.', (user.username, request.user.username))
    if settings.AUTH_WORKFLOW:
        send_notify_confirmation(request, user)


@receiver(user_rejected)
def user_rejected_handler(sender, **kwargs):
    '''
    Gets notified when a user was rejected by a manager.
    '''
    request = kwargs['request']
    user = kwargs['user']

    logger.info('user \'%s\' rejected by \'%s\'.', (user.username, request.user.username))
    if settings.AUTH_WORKFLOW:
        send_notify_rejection(request, user)


@receiver(user_activated)
def user_activated_handler(sender, **kwargs):
    '''
    Gets notified when a user was activated by an admin.
    '''
    request = kwargs['request']
    user = kwargs['user']

    logger.info('user \'%s\' activated by \'%s\'.', (user.username, request.user.username))
    if settings.AUTH_WORKFLOW:
        send_notify_activation(request, user)
        send_activation(request, user)


@receiver(user_disabled)
def user_disabled_handler(sender, **kwargs):
    '''
    Gets notified when a user was disabled by a manager.
    '''
    request = kwargs['request']
    user = kwargs['user']

    logger.info('user \'%s\' disabled by \'%s\'.', (user.username, request.user.username))


@receiver(user_enabled)
def user_enabled_handler(sender, **kwargs):
    '''
    Gets notified when a user was disabled by a manager.
    '''
    request = kwargs['request']
    user = kwargs['user']

    logger.info('user \'%s\' enabled by \'%s\'.', (user.username, request.user.username))


@receiver(password_changed)
@receiver(password_reset)
def password_changed_handler(sender, **kwargs):
    '''
    Gets notified when a user was disabled by a manager.
    '''
    request = kwargs['request']
    user = kwargs['user']

    logger.info('user \'%s\' changed his/her password.', user.username)
    if settings.AUTH_WORKFLOW == 'confirmation':
        send_notify_password_changed(request, user)



from django.apps import apps
from django.core.cache import cache
from django.db.models.signals import post_delete, post_save
from django.dispatch import receiver

from .models import Column, Schema, Table


@receiver(post_save, sender=Schema)
def schema_updated_handler(sender, **kwargs):
    if not kwargs.get('raw'):
        if apps.is_installed('daiquiri.oai'):
            from daiquiri.oai.utils import update_records
            update_records('schema', kwargs['instance'])

        if apps.is_installed('daiquiri.datalink'):
            from daiquiri.datalink.utils import update_links
            update_links('schema', kwargs['instance'])

        if apps.is_installed('daiquiri.tap'):
            from daiquiri.tap.utils import update_schema
            update_schema(kwargs['instance'])


@receiver(post_delete, sender=Schema)
def schema_deleted_handler(sender, **kwargs):
    if not kwargs.get('raw'):
        if apps.is_installed('daiquiri.oai'):
            from daiquiri.oai.utils import delete_records
            delete_records('schema', kwargs['instance'])

        if apps.is_installed('daiquiri.datalink'):
            from daiquiri.datalink.utils import delete_links
            delete_links('schema', kwargs['instance'])

        if apps.is_installed('daiquiri.tap'):
            from daiquiri.tap.utils import delete_schema
            delete_schema(kwargs['instance'])


@receiver(post_save, sender=Table)
def table_updated_handler(sender, **kwargs):
    table = kwargs['instance']

    if not kwargs.get('raw'):
        if apps.is_installed('daiquiri.oai'):
            from daiquiri.oai.utils import update_records
            update_records('table', table)

        if apps.is_installed('daiquiri.datalink'):
            from daiquiri.datalink.utils import update_links
            update_links('table', table)

        if apps.is_installed('daiquiri.tap'):
            from daiquiri.tap.utils import update_table
            update_table(table)


@receiver(post_delete, sender=Table)
def table_deleted_handler(sender, **kwargs):
    table = kwargs['instance']

    if not kwargs.get('raw'):
        if apps.is_installed('daiquiri.oai'):
            from daiquiri.oai.utils import delete_records
            delete_records('table', table)

        if apps.is_installed('daiquiri.datalink'):
            from daiquiri.datalink.utils import delete_links
            delete_links('table', table)

        if apps.is_installed('daiquiri.tap'):
            from daiquiri.tap.utils import delete_table
            delete_table(table)


@receiver(post_save, sender=Column)
def column_updated_handler(sender, **kwargs):
    column = kwargs['instance']

    # clear cached queryparser
    cache.delete('processor')

    if not kwargs.get('raw'):
        if apps.is_installed('daiquiri.tap'):
            from daiquiri.tap.utils import update_column
            update_column(column)


@receiver(post_delete, sender=Column)
def column_deleted_handler(sender, **kwargs):
    column = kwargs['instance']

    if not kwargs.get('raw'):
        if apps.is_installed('daiquiri.tap'):
            from daiquiri.tap.utils import delete_column
            delete_column(column)



import pytest

from django.urls import reverse

users = (
    ('admin', 'admin'),
    ('manager', 'manager'),
    ('user', 'user'),
    ('anonymous', None),
)

status_map = {
    'html': {
        'admin': 200, 'manager': 200, 'user': 200, 'anonymous': 302
    },
    'html_a': {
        'admin': 200, 'manager': 200, 'user': 200, 'anonymous': 302
    },
    'html_a_a': {
        'admin': 403, 'manager': 200, 'user': 403, 'anonymous': 302
    },
    'html_a_b': {
        'admin': 404, 'manager': 404, 'user': 404, 'anonymous': 404
    }
}


@pytest.mark.parametrize(('username', 'password'), users)
def test_html(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse('files:file', kwargs={
        'file_path': 'html/'
    })
    response = client.get(url)
    assert response.status_code == status_map['html'][username]


@pytest.mark.parametrize(('username', 'password'), users)
def test_html_a(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse('files:file', kwargs={
        'file_path': 'html/a/'
    })
    response = client.get(url)
    assert response.status_code == status_map['html_a'][username]


@pytest.mark.parametrize(('username', 'password'), users)
def test_html_a_a(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse('files:file', kwargs={
        'file_path': 'html/a/a/'
    })
    response = client.get(url)
    assert response.status_code == status_map['html_a_a'][username]


@pytest.mark.parametrize(('username', 'password'), users)
def test_html_a_b(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse('files:file', kwargs={
        'file_path': 'html/a/b/'
    })
    response = client.get(url)
    assert response.status_code == status_map['html_a_b'][username]



#from django.conf import settings
from django.db.models.signals import post_delete, pre_save
from django.dispatch import receiver

from .models import DownloadJob, QueryJob


@receiver(pre_save, sender=QueryJob)
def query_job_updated_handler(sender, **kwargs):
    instance = kwargs['instance']

    try:
        QueryJob.objects.get(pk=instance.pk).rename_table(instance.table_name)
    except QueryJob.DoesNotExist:
        pass


@receiver(post_delete, sender=QueryJob)
def query_job_deleted_handler(sender, **kwargs):
    kwargs['instance'].drop_table()


@receiver(post_delete, sender=DownloadJob)
def download_job_deleted_handler(sender, **kwargs):
    kwargs['instance'].delete_file()



# Generated by Django 2.1.3 on 2018-11-05 16:59

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_contact', '0004_meta'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='contactmessage',
            options={'ordering': ('-created', 'author'), 'verbose_name': 'Contact message', 'verbose_name_plural': 'Contact messages'},
        ),
    ]



import json
from uuid import UUID


class UUIDJSONEncoder(json.JSONEncoder):

    def default(self, obj):
        if isinstance(obj, UUID):
            return obj.hex

        return json.JSONEncoder.default(self, obj)



from django.conf import settings

from celery import Celery
from kombu import Exchange, Queue


def get_celery_app():
    app = Celery(settings.DAIQUIRI_APP)

    # configure celery and add apps
    app.conf.broker_url = settings.CELERY_BROKER_URL
    app.conf.worker_prefetch_multiplier = 1
    app.conf.task_acks_late = True

    # configure queues
    queues = [
        Queue('default', Exchange('default'), routing_key='default'),
        Queue('download', Exchange('download'), routing_key='download')
    ]
    for queue in settings.QUERY_QUEUES:
        queue_name = exchange_name = routing_key = 'query.{key}'.format(**queue)
        queue = Queue(queue_name, Exchange(exchange_name), routing_key=routing_key)
        queues.append(queue)

    app.conf.task_default_queue = 'default'
    app.conf.task_queues = queues

    # load task modules from all registered Django app configs
    app.autodiscover_tasks()

    return app



from importlib.util import find_spec
from pathlib import Path

import daiquiri.core.env as env
from django.utils.translation import gettext_lazy as _

try:
    CONFIG_DIR = Path(find_spec("config").origin).parent
    BASE_DIR = CONFIG_DIR.parent
    DAIQUIRI_APP = BASE_DIR.name.replace("-", "_")
except AttributeError:
    BASE_DIR = Path().cwd()
    DAIQUIRI_APP = None

BASE_URL = env.get_url("BASE_URL", "/")

DEBUG = env.get_bool("DEBUG")

SECRET_KEY = env.get("SECRET_KEY")

ALLOWED_HOSTS = env.get_list("ALLOWED_HOSTS", ["localhost", "127.0.0.1", "::1"])

if env.get_bool("PROXY"):
    USE_X_FORWARDED_HOST = True
    SECURE_PROXY_SSL_HEADER = ("HTTP_X_FORWARDED_PROTO", "https")

INTERNAL_IPS = env.get_list("INTERNAL_IPS", ["127.0.0.1"])

ADMINS = env.get_email_list("ADMINS")

SITE_ID = 1

ROOT_URLCONF = "config.urls"

WSGI_APPLICATION = "config.wsgi.application"

DATABASES = {
    "default": env.get_database("app"),
    "data": env.get_database("data"),
    "tap": env.get_database("data"),
    "oai": env.get_database("data"),
}

DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"

if DATABASES["tap"].get("ENGINE") == "django.db.backends.postgresql":
    DATABASES["tap"]["OPTIONS"] = {
        "options": "-c search_path={}".format(env.get("TAP_SCHEMA", "tap_schema"))
    }
elif DATABASES["tap"].get("ENGINE") == "django.db.backends.mysql":
    DATABASES["tap"]["NAME"] = env.get("TAP_SCHEMA", "tap_schema")

if DATABASES["oai"].get("ENGINE") == "django.db.backends.postgresql":
    DATABASES["oai"]["OPTIONS"] = {
        "options": "-c search_path={}".format(env.get("OAI_SCHEMA", "oai_schema"))
    }
elif DATABASES["oai"].get("ENGINE") == "django.db.backends.mysql":
    DATABASES["oai"]["NAME"] = env.get("OAI_SCHEMA", "oai_schema")

ADAPTER_DATABASE = env.get_database_adapter()
ADAPTER_DOWNLOAD = env.get_download_adapter()

if DATABASES["data"].get("ENGINE") == "django.db.backends.postgresql":
    USER_TABLESPACE = env.get("USER_TABLESPACE", "pg_default")
else:
    USER_TABLESPACE = None

DATABASE_ROUTERS = [
    "daiquiri.core.routers.TapRouter",
    "daiquiri.core.routers.OaiRouter",
    "daiquiri.core.routers.DataRouter",
]

DJANGO_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "django.contrib.sites",
    "django.forms",
]

ADDITIONAL_APPS = [
    "rest_framework",
    "rest_framework.authtoken",
    "django_filters",
    "django_extensions",
    "markdown",
    "widget_tweaks",
    "allauth",
    "allauth.account",
    "allauth.socialaccount",
    "rules",
    "django_sendfile",
    "honeypot",
]

MIDDLEWARE = [
    "daiquiri.core.middleware.MultipleProxyMiddleware",
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.locale.LocaleMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    "django.contrib.sites.middleware.CurrentSiteMiddleware",
]

TEMPLATES_DIR = BASE_DIR / "templates/"
TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [TEMPLATES_DIR] if TEMPLATES_DIR.exists() else [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
                "django_settings_export.settings_export",
            ],
        },
    },
]

LANGUAGE_CODE = "en-us"

TIME_ZONE = env.get("TIME_ZONE", "UTC")

LOCALE_PATHS = (BASE_DIR / "locale/",)

LANGUAGES = (("en", _("English")),)

USE_I18N = True

USE_TZ = True

LOGIN_URL = BASE_URL + "accounts/login/"
LOGIN_REDIRECT_URL = BASE_URL
LOGOUT_URL = BASE_URL + "accounts/logout/"
LOGOUT_REDIRECT_URL = BASE_URL

CSRF_COOKIE_PATH = BASE_URL
LANGUAGE_COOKIE_PATH = BASE_URL
SESSION_COOKIE_PATH = BASE_URL

MEDIA_URL = BASE_URL + "media/"
MEDIA_ROOT = BASE_DIR / "media_root/"

STATIC_URL = BASE_URL + "static/"
STATIC_ROOT = BASE_DIR / "static_root/"

STATICFILES_FINDERS = (
    "django.contrib.staticfiles.finders.FileSystemFinder",
    "django.contrib.staticfiles.finders.AppDirectoriesFinder",
)

FIXTURE_DIRS = (BASE_DIR / "fixtures",)

FORM_RENDERER = "daiquiri.core.forms.DaiquiriFormRenderer"

AUTHENTICATION_BACKENDS = (
    "rules.permissions.ObjectPermissionBackend",
    "django.contrib.auth.backends.ModelBackend",
    "allauth.account.auth_backends.AuthenticationBackend",
)

ACCOUNT_LOGOUT_ON_GET = False
ACCOUNT_LOGOUT_REDIRECT_URL = BASE_URL
ACCOUNT_ADAPTER = "daiquiri.auth.adapter.DaiquiriAccountAdapter"
ACCOUNT_SIGNUP_FORM_CLASS = "daiquiri.auth.forms.SignupForm"
ACCOUNT_USER_DISPLAY = "daiquiri.auth.utils.get_full_name"
ACCOUNT_USERNAME_VALIDATORS = "daiquiri.auth.validators.username_validators"
ACCOUNT_USERNAME_MIN_LENGTH = 4
ACCOUNT_PASSWORD_MIN_LENGTH = 4
ACCOUNT_EMAIL_MAX_LENGTH = 190
ACCOUNT_EMAIL_REQUIRED = True
ACCOUNT_EMAIL_CONFIRMATION_EXPIRE_DAYS = 7
# 'mandatory' or 'optional'
ACCOUNT_EMAIL_VERIFICATION = env.get("ACCOUNT_EMAIL_VERIFICATION", "mandatory")
ACCOUNT_LOGIN_ON_EMAIL_CONFIRMATION = True

SOCIALACCOUNT_ADAPTER = "daiquiri.auth.adapter.DaiquiriSocialAccountAdapter"
SOCIALACCOUNT_AUTO_SIGNUP = False

REST_FRAMEWORK = {
    "DEFAULT_THROTTLE_CLASSES": ("rest_framework.throttling.ScopedRateThrottle",),
    "DEFAULT_THROTTLE_RATES": {"query.create": "10/second"},
}

SETTINGS_EXPORT = [
    "ACCOUNT_LOGOUT_ON_GET",
    "AUTH_WORKFLOW",
    "AUTH_TERMS_OF_USE",
    "LOGIN_URL",
    "LOGOUT_URL",
    "METADATA_COLUMN_PERMISSIONS",
    "SITE_PUBLISHER",
    "SITE_URL",
    "SITE_TITLE",
    "STATS_RESOURCE_TYPES",
]

DEFAULT_FROM_EMAIL = env.get("DEFAULT_FROM_EMAIL", "info@example.com")
EMAIL_BACKEND = env.get(
    "EMAIL_BACKEND", "django.core.mail.backends.console.EmailBackend"
)
EMAIL_REPLY_TO = env.get("EMAIL_REPLY_TO")
EMAIL_HOST = env.get("EMAIL_HOST")
EMAIL_HOST_USER = env.get("EMAIL_HOST_USER")
EMAIL_HOST_PASSWORD = env.get("EMAIL_HOST_PASSWORD")
EMAIL_PORT = env.get("EMAIL_PORT", "25")
EMAIL_USE_TLS = env.get_bool("EMAIL_USE_TLS")

SENDFILE_BACKEND = env.get("SENDFILE_BACKEND", "django_sendfile.backends.simple")
SENDFILE_ROOT = env.get("SENDFILE_ROOT")
SENDFILE_URL = env.get("SENDFILE_URL")

MEMCACHE_KEY_PREFIX = env.get("MEMCACHE_KEY_PREFIX")
if MEMCACHE_KEY_PREFIX:
    CACHES = {
        "default": {
            "BACKEND": "django.core.cache.backends.memcached.MemcachedCache",
            "LOCATION": "127.0.0.1:11211",
            "KEY_PREFIX": MEMCACHE_KEY_PREFIX,
        }
    }

CELERY_BROKER_URL = env.get("CELERY_BROKER_URL", "amqp://")
CELERY_BIN = env.get("CELERY_BIN", "celery")
CELERY_PIDFILE_PATH = env.get_abspath("CELERY_PIDFILE_PATH")
CELERY_LOG_LEVEL = env.get("CELERY_LOG_LEVEL", "INFO")
CELERY_LOG_PATH = env.get_abspath("CELERY_LOG_PATH")

HONEYPOT_ENABLED = True
HONEYPOT_FIELD_NAME = "Phone"
HONEYPOT_FIELD_VALUE = ""
HONEYPOT_FIELD_HIDDEN = True



# Generated by Django 2.1.3 on 2018-11-05 16:59

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_files', '0003_python3'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='directory',
            options={'ordering': ('path',), 'verbose_name': 'Directory', 'verbose_name_plural': 'Directory'},
        ),
    ]



from django.contrib.auth.models import User
from django.core.management.base import BaseCommand


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument('username', help='Username of the user to promote.')
        parser.add_argument('--demote', action='store_true', default=False, help='Demote instead of promote.')

    def handle(self, *args, **options):
        is_admin = False if options['demote'] else True

        user = User.objects.get(username=options['username'])
        user.is_superuser = is_admin
        user.is_staff = is_admin
        user.save()



# -*- coding: utf-8 -*-
# Generated by Django 1.11.10 on 2018-02-16 16:50
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_query', '0015_queryarchivejob'),
    ]

    operations = [
        migrations.RenameField(
            model_name='queryjob',
            old_name='database_name',
            new_name='schema_name',
        ),
    ]



# see also https://strongarm.io/blog/multiple-databases-in-django/


class DaiquiriRouter:

    def db_for_read(self, model, **hints):
        if model._meta.app_label in self.app_labels:
            return self.db
        return None

    def db_for_write(self, model, **hints):
        if model._meta.app_label in self.app_labels:
            return self.db
        return None

    def allow_relation(self, obj1, obj2, **hints):
        # Allow any relation between two models that are both in the `app_label` app.
        if obj1._meta.app_label in self.app_labels or obj2._meta.app_label in self.app_labels:
            return True

        # No opinion if neither object is in the Example app (defer to default or other routers).
        elif set(self.app_labels).intersection([obj1._meta.app_label, obj2._meta.app_label]):
            return None

        # Block relationship if one object is in the `app_label` and the other isn't.
        return None

    def allow_migrate(self, db, app_label, model_name=None, **hints):

        if app_label in self.app_labels:
            # The `app_label` should be migrated only on the `db` database.
            return db == self.db

        elif db == self.db:
            # Ensure that all other apps don't get migrated on the example_db database.
            return False

        # No opinion for all other scenarios
        return None


class TapRouter(DaiquiriRouter):

    db = 'tap'
    app_labels = ['daiquiri_datalink', 'daiquiri_tap']


class OaiRouter(DaiquiriRouter):

    db = 'oai'
    app_labels = ['daiquiri_oai']


class DataRouter:

    def db_for_read(self, model, **hints):
        return None

    def db_for_write(self, model, **hints):
        return None

    def allow_relation(self, obj1, obj2, **hints):
        return None

    def allow_migrate(self, db, app_label, model_name=None, **hints):
        # prevent migrations for the `data` db
        if db == 'data':
            return False
        return None



import logging
import os
from collections import OrderedDict

from django.conf import settings
from django.contrib.auth.models import Group
from django.db import models
from django.db.utils import DataError, InternalError, OperationalError, ProgrammingError
from django.utils.functional import cached_property
from django.utils.timezone import now
from django.utils.translation import gettext_lazy as _

from rest_framework.exceptions import ValidationError

from daiquiri.core.adapter import DatabaseAdapter, DownloadAdapter
from daiquiri.core.constants import ACCESS_LEVEL_CHOICES
from daiquiri.core.utils import get_date_display
from daiquiri.files.utils import check_file
from daiquiri.jobs.managers import JobManager
from daiquiri.jobs.models import Job
from daiquiri.stats.models import Record

from .managers import ExampleManager, QueryJobManager
from .process import (
    check_number_of_active_jobs,
    check_permissions,
    check_quota,
    process_display_columns,
    process_query,
    process_query_language,
    process_queue,
    process_response_format,
    process_schema_name,
    process_table_name,
    process_user_columns,
    translate_query,
)
from .tasks import (
    abort_database_query_task,
    create_download_archive_task,
    create_download_table_task,
    drop_database_table_task,
    rename_database_table_task,
    run_database_ingest_task,
    run_database_query_task,
)
from .utils import get_format_config, get_job_columns, get_query_language_label

logger = logging.getLogger(__name__)
query_logger = logging.getLogger('query')


class QueryJob(Job):

    objects = QueryJobManager()

    schema_name = models.CharField(max_length=256)
    table_name = models.CharField(max_length=256)
    queue = models.CharField(max_length=16, blank=True)

    query_language = models.CharField(max_length=16, blank=True)
    query = models.TextField(blank=True)
    native_query = models.TextField(blank=True)
    actual_query = models.TextField(blank=True)

    queue = models.CharField(max_length=16, blank=True)
    nrows = models.BigIntegerField(null=True, blank=True)
    size = models.BigIntegerField(null=True, blank=True)

    metadata = models.JSONField(null=True, blank=True)
    uploads = models.JSONField(null=True, blank=True)

    pid = models.IntegerField(null=True, blank=True)

    class Meta:
        ordering = ('start_time', )

        verbose_name = _('QueryJob')
        verbose_name_plural = _('QueryJobs')

    @property
    def phase_label(self):
        return self.get_phase_display()

    @property
    def query_language_label(self):
        return get_query_language_label(self.query_language)

    @property
    def creation_time_label(self):
        return get_date_display(self.creation_time)

    @property
    def start_time_label(self):
        return get_date_display(self.start_time)

    @property
    def end_time_label(self):
        return get_date_display(self.end_time)

    @property
    def parameters(self):
        return {
            'schema_name': self.schema_name,
            'table_name': self.table_name,
            'query_language': self.query_language,
            'query': self.query,
            'native_query': self.native_query,
            'actual_query': self.actual_query,
            'queue': self.queue,
            'nrows': self.nrows,
            'size': self.size
        }

    @property
    def formats(self):
        return OrderedDict((item['key'], item['content_type']) for item in settings.QUERY_DOWNLOAD_FORMATS)

    @property
    def result_status(self):
        return 'OK' if self.max_records is None else 'OVERFLOW'

    @property
    def quote(self):
        return None

    @property
    def time_queue(self):
        if self.start_time and self.creation_time:
            return (self.start_time - self.creation_time).total_seconds()
        else:
            return None

    @property
    def time_query(self):
        if self.end_time and self.start_time:
            return (self.end_time - self.start_time).total_seconds()
        else:
            return None

    @property
    def timeout(self):
        if self.queue:
            return next(queue['timeout'] for queue in settings.QUERY_QUEUES if queue['key'] == self.queue)
        else:
            return 10

    @cached_property
    def column_names(self):
        return [column['name'] for column in self.metadata['columns']]

    def process(self, upload=False):
        # log the query to the query log
        query_logger.info('"%s" %s %s', self.query, self.query_language, self.owner or 'anonymous')

        # check quota and number of active jobs
        check_quota(self)
        check_number_of_active_jobs(self)

        # process schema_name, table_name and response format
        self.schema_name = process_schema_name(self.owner, self.schema_name)
        self.table_name = process_table_name(self.table_name)
        self.response_format = process_response_format(self.response_format)

        if upload:
            self.query = ''
            self.query_language = ''
            self.queue = ''
            self.execution_duration = 0.0

        else:
            self.query_language = process_query_language(self.owner, self.query_language)
            self.queue = process_queue(self.owner, self.queue)
            self.response_format = process_response_format(self.response_format)

            # set the execution_duration to the queues timeout
            self.execution_duration = self.timeout

            # log the input query to the debug log
            logger.debug('query = "%s"', self.query)

            # translate the query from adql
            translated_query = translate_query(self.query_language, self.query)

            # log the translated query to the debug log
            logger.debug('translated_query = "%s"', translated_query)

            processor = process_query(translated_query)

            # log the processor output to the debug log
            logger.debug('native_query = "%s"', processor.query)
            logger.debug('processor.keywords = %s', processor.keywords)
            logger.debug('processor.tables = %s', processor.tables)
            logger.debug('processor.columns = %s', processor.columns)
            logger.debug('processor.functions = %s', processor.functions)

            # check permissions
            permission_messages = check_permissions(
                self.owner,
                processor.keywords,
                processor.tables,
                processor.columns,
                processor.functions
            )
            if permission_messages:
                raise ValidationError({
                    'query': {
                        'messages': permission_messages,
                    }
                })

            # initialize metadata and store map of aliases
            self.metadata = {
                'display_columns': process_display_columns(processor.display_columns),
                'user_columns': process_user_columns(self, processor.tables),
                'tables': processor.tables
            }

            # get the native query from the processor (without trailing semicolon)
            self.native_query = processor.query.rstrip(';')

            # set clean flag
            self.is_clean = True

    def run(self):
        if not self.is_clean:
            raise Exception('job.process() was not called.')

        if self.phase == self.PHASE_PENDING:
            self.phase = self.PHASE_QUEUED
            self.save()

            # start the submit_query task in a synchronous or asuncronous way
            job_id = str(self.id)
            if not settings.ASYNC:
                logger.info('job %s submitted (sync)', self.id)
                run_database_query_task.apply((job_id, ), task_id=job_id, throw=True)

            else:
                queue = f'query_{self.queue}'
                logger.info('job %s submitted (async, queue=%s)', self.id, queue)
                run_database_query_task.apply_async((job_id, ), task_id=job_id, queue=queue)

        else:
            raise ValidationError({
                'phase': ['Job is not PENDING.']
            })

    def run_sync(self):
        adapter = DatabaseAdapter()

        adapter.create_user_schema_if_not_exists(self.schema_name)

        self.actual_query = adapter.build_query(
            self.schema_name,
            self.table_name,
            self.native_query,
            settings.QUERY_SYNC_TIMEOUT,
            self.max_records,
        )

        adapter.submit_query(self.actual_query)

        # create a stats record for this job
        Record.objects.create(
            time=now(),
            resource_type='QUERY',
            resource={
                'job_id': None,
                'job_type': self.job_type,
            },
            client_ip=self.client_ip,
            user=self.owner,
            size=adapter.fetch_size(self.schema_name, self.table_name)
        )

        try:

            yield from DownloadAdapter().generate(
                    'votable',
                    get_job_columns(self),
                    sources=self.metadata.get("sources", []),
                    schema_name=self.schema_name,
                    table_name=self.table_name,
                    nrows=self.nrows,
                    query_status=self.result_status,
                    query=self.native_query,
                    query_language=self.query_language)

            self.drop_uploads()
            self.drop_table()

        except (OperationalError, ProgrammingError, InternalError, DataError) as e:
            raise StopIteration from e

    def ingest(self, file_path):
        if self.phase == self.PHASE_PENDING:
            self.phase = self.PHASE_QUEUED
            self.save()

            if not settings.ASYNC:
                run_database_ingest_task.apply((self.id, file_path), throw=True)
            else:
                run_database_ingest_task.apply_async((self.id, file_path), queue='download')

        else:
            raise ValidationError({
                'phase': ['Job is not PENDING.']
            })

    def abort(self):
        current_phase = self.phase
        if current_phase in self.PHASE_ACTIVE:
            # next, set the phase to ABORTED
            self.phase = self.PHASE_ABORTED
            self.save()

            # finally, abort query, this will trigger OperationalError in the run_query task
            if current_phase == self.PHASE_EXECUTING:
                self.abort_query()

    def archive(self):
        self.abort()
        self.drop_table()
        self.drop_uploads()
        self.phase = self.PHASE_ARCHIVED
        self.nrows = None
        self.size = None
        self.save()

    def rename_table(self, new_table_name):
        if self.table_name != new_table_name:
            self.metadata['name'] = new_table_name
            self.save()

            task_args = (self.schema_name, self.table_name, new_table_name)

            if not settings.ASYNC:
                rename_database_table_task.apply(task_args, throw=True)
            else:
                rename_database_table_task.apply_async(task_args)

    def drop_table(self):
        task_args = (self.schema_name, self.table_name)

        if not settings.ASYNC:
            drop_database_table_task.apply(task_args, throw=True)
        else:
            drop_database_table_task.apply_async(task_args)

    def drop_uploads(self):
        if self.uploads:
            for table_name, file_path in self.uploads.items():
                task_args = (settings.TAP_UPLOAD, table_name)

                if not settings.ASYNC:
                    drop_database_table_task.apply(task_args, throw=True)
                else:
                    drop_database_table_task.apply_async(task_args)

    def abort_query(self):
        task_args = (self.pid, )

        if not settings.ASYNC:
            abort_database_query_task.apply(task_args, throw=True)
        else:
            abort_database_query_task.apply_async(task_args)

    def stream(self, format_key):
        if self.phase == self.PHASE_COMPLETED:
            return DownloadAdapter().generate(
                format_key,
                self.metadata.get('columns', []),
                sources=self.metadata.get('sources', []),
                schema_name=self.schema_name,
                table_name=self.table_name,
                nrows=self.nrows,
                query_status=self.result_status,
                query=self.query,
                query_language=self.query_language
            )
        else:
            raise ValidationError({
                'phase': ['Job is not COMPLETED.']
            })

    def rows(self, column_names, ordering, page, page_size, search, filters):
        if self.phase == self.PHASE_COMPLETED:
            # check if the columns are actually in the jobs table
            errors = {}

            for column_name in column_names:
                if column_name not in self.column_names:
                    errors[column_name] = _('Column not found.')

            if errors:
                raise ValidationError(errors)

            # get database adapter
            adapter = DatabaseAdapter()

            try:
                # query the database for the total number of rows
                count = adapter.count_rows(self.schema_name, self.table_name, column_names, search, filters)

                # query the paginated rowset
                rows = adapter.fetch_rows(self.schema_name, self.table_name, column_names,
                                          ordering, page, page_size, search, filters)

                # flatten the list if only one column is retrieved
                if len(column_names) == 1:
                    return count, [element for row in rows for element in row]
                else:
                    return count, rows

            except ProgrammingError:
                return 0, []

        else:
            raise ValidationError({
                'phase': ['Job is not COMPLETED.']
            })

    def columns(self):
        if self.metadata:
            return self.metadata.get('columns', [])
        else:
            return []


class DownloadJob(Job):

    objects = JobManager()

    query_job = models.ForeignKey(
        QueryJob, related_name='downloads', on_delete=models.CASCADE,
        verbose_name=_('QueryJob'),
        help_text=_('QueryJob this DownloadJob belongs to.')
    )
    format_key = models.CharField(
        max_length=32,
        verbose_name=_('Format key'),
        help_text=_('Format key for this download.')
    )

    class Meta:
        ordering = ('start_time', )

        verbose_name = _('DownloadJob')
        verbose_name_plural = _('DownloadJobs')

    @cached_property
    def file_path(self):
        if not self.owner:
            username = 'anonymous'
        else:
            username = self.owner.username

        format_config = get_format_config(self.format_key)

        if format_config:
            directory_name = os.path.join(settings.QUERY_DOWNLOAD_DIR, username)
            return os.path.join(directory_name, '{}.{}'.format(self.query_job.table_name, format_config['extension']))
        else:
            return None

    def process(self):
        if self.query_job.phase == self.PHASE_COMPLETED:
            self.owner = self.query_job.owner
        else:
            raise ValidationError({
                'phase': ['Job is not COMPLETED.']
            })

        # set clean flag
        self.is_clean = True

    def run(self):
        if not self.is_clean:
            raise Exception('download_job.process() was not called.')

        if self.phase == self.PHASE_PENDING:
            self.phase = self.PHASE_QUEUED
            self.save()

            download_id = str(self.id)
            if not settings.ASYNC:
                logger.info('download_job %s submitted (sync)', download_id)
                create_download_table_task.apply((download_id, ), task_id=download_id, throw=True)

            else:
                logger.info('download_job %s submitted (async, queue=download)', download_id)
                create_download_table_task.apply_async((download_id, ), task_id=download_id, queue='download')

        else:
            raise ValidationError({
                'phase': ['Job is not PENDING.']
            })

    def delete_file(self):
        try:
            if self.file_path is not None:
                os.remove(self.file_path)
        except OSError:
            pass


class QueryArchiveJob(Job):

    objects = JobManager()

    query_job = models.ForeignKey(
        QueryJob, related_name='archives', on_delete=models.CASCADE,
        verbose_name=_('QueryJob'),
        help_text=_('QueryJob this ArchiveJob belongs to.')
    )
    column_name = models.CharField(
        max_length=32,
        verbose_name=_('Column name'),
        help_text=_('Column name for this download.')
    )
    files = models.JSONField(
        verbose_name=_('Files'),
        help_text=_('List of files in the archive.'),
        default=list
    )

    class Meta:
        ordering = ('start_time', )

        verbose_name = _('QueryArchiveJob')
        verbose_name_plural = _('QueryArchiveJob')

    @cached_property
    def file_path(self):
        if not self.owner:
            username = 'anonymous'
        else:
            username = self.owner.username

        directory_name = os.path.join(settings.QUERY_DOWNLOAD_DIR, username)
        return os.path.join(directory_name, f'{self.query_job.table_name}.{self.column_name}.zip')

    def process(self):
        if self.query_job.phase == self.PHASE_COMPLETED:
            self.owner = self.query_job.owner
        else:
            raise ValidationError({
                'phase': ['Job is not COMPLETED.']
            })

        if not self.column_name:
            raise ValidationError({
                'column_name': [_('This field may not be blank.')]
            })

        if self.column_name not in self.query_job.column_names:
            raise ValidationError({
                'column_name': [_('Unknown column "%s".') % self.column_name]
            })

        # get database adapter and query the paginated rowset
        rows = DatabaseAdapter().fetch_rows(self.query_job.schema_name, self.query_job.table_name,
                                            [self.column_name], page_size=0)

        # prepare list of files for this job
        files = []
        for row in rows:
            file_path = row[0]

            # append the file to the list of files  if it exists
            if file_path and check_file(self.owner, file_path):
                files.append(file_path)
            else:
                raise ValidationError({
                    'files': [_('One or more of the files cannot be found.')]
                })

        # set files for this job
        self.files = files

        # set clean flag
        self.is_clean = True

    def run(self):
        if not self.is_clean:
            raise Exception('download_job.process() was not called.')

        if self.phase == self.PHASE_PENDING:
            self.phase = self.PHASE_QUEUED
            self.save()

            archive_id = str(self.id)
            if not settings.ASYNC:
                logger.info('archive_job %s submitted (sync)', archive_id)
                create_download_archive_task.apply((archive_id, ), task_id=archive_id, throw=True)

            else:
                logger.info('archive_job %s submitted (async, queue=download)', archive_id)
                create_download_archive_task.apply_async((archive_id, ), task_id=archive_id, queue='download')

        else:
            raise ValidationError({
                'phase': ['Job is not PENDING.']
            })


    def delete_file(self):
        try:
            os.remove(self.file_path)
        except OSError:
            pass


class Example(models.Model):

    order = models.IntegerField(
        null=True, blank=True,
        verbose_name=_('Order'),
        help_text=_('Position in lists.')
    )
    name = models.CharField(
        max_length=256,
        verbose_name=_('Name'),
        help_text=_('Identifier of the example.')
    )
    description = models.TextField(  # noqa: DJ001
        null=True, blank=True,
        verbose_name=_('Description'),
        help_text=_('A brief description of the example to be displayed in the user interface.')
    )
    query_language = models.CharField(
        max_length=16,
        verbose_name=_('Query language'),
        help_text=_('The query language for this example.')
    )
    query_string = models.TextField(
        verbose_name=_('Query string'),
        help_text=_('The query string (SQL) for this example.')
    )
    access_level = models.CharField(
        max_length=8, choices=ACCESS_LEVEL_CHOICES,
        verbose_name=_('Access level')
    )
    groups = models.ManyToManyField(
        Group, blank=True,
        verbose_name=_('Groups'),
        help_text=_('The groups which have access to the examples.')
    )

    objects = ExampleManager()

    class Meta:
        ordering = ('order',)

        verbose_name = _('Example')
        verbose_name_plural = _('Examples')

    def __str__(self):
        return self.name

    @property
    def query_language_label(self):
        return get_query_language_label(self.query_language)



from django.urls import reverse

from allauth.account.models import EmailAddress

from ..models import Profile


def test_login(db, client):
    url = reverse('account_login')
    response = client.post(url, {
        'login': 'user',
        'password': 'user'
    })
    assert response.status_code == 302
    assert response.url == reverse('home')


def test_login_unverified(db, client):
    email = EmailAddress.objects.filter(user__username='user').first()
    email.verified = False
    email.save()

    url = reverse('account_login')
    response = client.post(url, {
        'login': 'user',
        'password': 'user'
    })

    # check that the signup redirects to the pending page or the confirm email page
    assert response.status_code == 302
    assert response.url == reverse('account_email_verification_sent')


def test_invalid(db, client):
    url = reverse('account_login')
    response = client.post(url, {
        'login': 'invalid',
        'password': 'invalid'
    })
    assert response.status_code == 200


def test_logout(db, client):
    url = reverse('account_logout')
    response = client.post(url)
    assert response.status_code == 302
    assert response.url == reverse('home')


def test_signup_get(db, client):
    url = reverse('account_signup')
    response = client.get(url)
    assert response.status_code == 200


def test_signup_post_email_exists_verified(db, client):
    url = reverse('account_signup')
    response = client.post(url, {
        'email': 'user@example.com',
        'username': 'user2',
        'first_name': 'Tanja',
        'last_name': 'Test',
        'password1': 'testing',
        'password2': 'testing'
    })

   # check that the signup redirects to the pending page or the confirm email page
    assert response.status_code == 302
    assert response.url == reverse('account_email_verification_sent')

    # check that a profile was not created
    assert Profile.objects.filter(user__username='user2').exists() is False


def test_signup_post_email_exists_unverified(db, client):
    email = EmailAddress.objects.filter(user__username='user').first()
    email.verified = False
    email.save()

    url = reverse('account_signup')
    response = client.post(url, {
        'email': 'user@example.com',
        'username': 'user2',
        'first_name': 'Tanja',
        'last_name': 'Test',
        'password1': 'testing',
        'password2': 'testing'
    })

   # check that the signup redirects to the pending page or the confirm email page
    assert response.status_code == 302
    assert response.url == reverse('account_email_verification_sent')

    # check that a profile was not created
    assert Profile.objects.filter(user__username='unverified_user_2').exists() is False


def test_signup_post_user_exists_unverified(db, client):
    email = EmailAddress.objects.filter(user__username='user').first()
    email.verified = False
    email.save()

    url = reverse('account_signup')
    response = client.post(url, {
        'email': 'user@example.com',
        'username': 'user',
        'first_name': 'Tanja',
        'last_name': 'Test',
        'password1': 'testing',
        'password2': 'testing'
    })

    # check that the signup returns 200 (with validation error)
    assert response.status_code == 200
    assert b'A user with that username already exists.' in response.content


def test_signup_post_user_exists_verified(db, client):
    url = reverse('account_signup')
    response = client.post(url, {
        'email': 'user@example2.com',
        'username': 'user',
        'first_name': 'Tanja',
        'last_name': 'Test',
        'password1': 'testing',
        'password2': 'testing'
    })

    # check that the signup returns 200 (with validation error)
    assert response.status_code == 200
    assert b'A user with that username already exists.' in response.content


def test_profile_get_for_user(db, client):
    client.login(username='user', password='user')

    url = reverse('account_profile')
    response = client.get(url)
    assert response.status_code == 200


def test_profile_get_for_anonymous(db, client):
    url = reverse('account_profile')
    response = client.get(url)
    assert response.status_code == 302
    assert response.url == reverse('account_login') + '?next=' + url


def test_profile_post_for_user(db, client):
    client.login(username='user', password='user')

    url = reverse('account_profile')
    response = client.post(url, {
        'first_name': 'Tanja',
        'last_name': 'Test',
    })
    assert response.status_code == 302
    assert response.url == reverse('home')


def test_profile_cancel_for_user(db, client):
    client.login(username='user', password='user')

    url = reverse('account_profile')
    response = client.post(url, {
        'cancel': True
    })
    assert response.status_code == 302
    assert response.url == reverse('home')


def test_profile_post_for_anonymous(db, client):
    url = reverse('account_profile')
    response = client.post(url, {
        'first_name': 'Tanja',
        'last_name': 'Test',
    })
    assert response.status_code == 302
    assert response.url == reverse('account_login') + '?next=' + url


def test_profile_json_get_for_user(db, client):
    client.login(username='user', password='user')

    url = reverse('account_profile_json')
    response = client.get(url)
    assert response.status_code == 200


def test_profile_json_get_for_anonymous(db, client):
    url = reverse('account_profile_json')
    response = client.get(url)
    assert response.status_code == 302
    assert response.url == reverse('account_login') + '?next=' + url


def test_token_get_for_user(db, client):
    client.login(username='user', password='user')

    url = reverse('account_token')
    response = client.get(url)
    assert response.status_code == 200


def test_token_get_for_anonymous(db, client):
    url = reverse('account_token')
    response = client.get(url)
    assert response.status_code == 302
    assert response.url == reverse('account_login') + '?next=' + url


def test_token_post_for_user(db, client):
    client.login(username='user', password='user')

    url = reverse('account_token')
    response = client.post(url)
    assert response.status_code == 200


def test_token_post_for_anonymous(db, client):
    url = reverse('account_token')
    response = client.post(url)
    assert response.status_code == 302
    assert response.url == reverse('account_login') + '?next=' + url



from django.conf import settings

from rest_framework import serializers

from daiquiri.core.serializers import JSONListField
from daiquiri.metadata.models import Schema, Table


class DublincoreSerializer(serializers.ModelSerializer):

    title = serializers.SerializerMethodField()
    description = serializers.SerializerMethodField()
    creators = JSONListField(default=[])
    contributors = JSONListField(default=[])
    subjects = serializers.ReadOnlyField(default=settings.SITE_SUBJECTS)
    publisher = serializers.ReadOnlyField(default=settings.SITE_PUBLISHER)
    date = serializers.ReadOnlyField(source='published')
    type = serializers.SerializerMethodField()
    identifier = serializers.SerializerMethodField()
    rights = serializers.ReadOnlyField(source='license')

    def get_title(self, obj):
        return obj.title or obj.name

    def get_description(self, obj):
        return obj.long_description or obj.description

    def get_type(self, obj):
        return 'Dataset'

    def get_identifier(self, obj):
        raise NotImplementedError()


class DublincoreSchemaSerializer(DublincoreSerializer):

    class Meta:
        model = Schema
        fields = (
            'title',
            'description',
            'creators',
            'contributors',
            'subjects',
            'publisher',
            'date',
            'type',
            'identifier',
            'rights'
        )

    def get_identifier(self, obj):
        return obj.doi or 'schemas/%i' % obj.pk


class DublincoreTableSerializer(DublincoreSerializer):

    class Meta:
        model = Table
        fields = (
            'title',
            'description',
            'creators',
            'contributors',
            'subjects',
            'publisher',
            'date',
            'type',
            'identifier',
            'rights'
        )

    def get_identifier(self, obj):
        return obj.doi or 'tables/%i' % obj.pk



# Generated by Django 2.1.4 on 2019-05-29 13:44

from django.db import migrations
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_metadata', '0025_add_published_updated'),
    ]

    operations = [
        migrations.AddField(
            model_name='schema',
            name='contributors',
            field=jsonfield.fields.JSONField(blank=True, null=True, verbose_name='Contributors'),
        ),
        migrations.AddField(
            model_name='schema',
            name='creators',
            field=jsonfield.fields.JSONField(blank=True, null=True, verbose_name='Creators'),
        ),
        migrations.AddField(
            model_name='table',
            name='contributors',
            field=jsonfield.fields.JSONField(blank=True, null=True, verbose_name='Contributors'),
        ),
        migrations.AddField(
            model_name='table',
            name='creators',
            field=jsonfield.fields.JSONField(blank=True, null=True, verbose_name='Creators'),
        ),
    ]



from django.apps import apps
from django.conf import settings
from django.urls import reverse

from daiquiri.core.vo import get_curation


def get_resource():
    return {
        'service': 'registry',
        'identifier': f'ivo://{settings.SITE_IDENTIFIER}/registry',
        'title': f'{settings.SITE_TITLE} Registry',
        'curation': get_curation(),
        'content': {
            'subjects': settings.REGISTRY_SUBJECTS,
            'type': 'Registry',
            'description': f'The publishing registry for {settings.SITE_IDENTIFIER}.',
            'referenceURL': settings.SITE_URL.rstrip('/') + reverse('registry:root')
        },
        'capabilities': get_capabilities(),
        'full': 'false',
        'managed_authority': settings.SITE_IDENTIFIER,
        'created': settings.SITE_CREATED,
        'updated': settings.SITE_UPDATED,
        'type': 'vg:Registry',
        'status': 'active',
        'tableset': get_tap_tableset()
    }


def get_authority_resource():
    return {
        'service': 'authority',
        'identifier': f'ivo://{settings.SITE_IDENTIFIER}',
        'title': settings.SITE_IDENTIFIER,
        'curation': get_curation(),
        'content': {
            'subjects': settings.REGISTRY_AUTHORITY_SUBJECTS,
            'type': 'Authority',
            'description': f'The authority for {settings.SITE_IDENTIFIER}.',
            'referenceURL': settings.SITE_URL.rstrip('/')
        },
        'capabilities': [],
        'managing_org': settings.SITE_PUBLISHER,
        'created': settings.SITE_CREATED,
        'updated': settings.SITE_UPDATED,
        'type': 'vg:Authority',
        'status': 'active'
    }


def get_web_resource():
    return {
        'service': 'web',
        'identifier': f'ivo://{settings.SITE_IDENTIFIER}/web',
        'title': settings.SITE_IDENTIFIER,
        'curation': get_curation(),
        'content': {
            'subjects': settings.REGISTRY_WEB_SUBJECTS,
            'type': 'Catalog',
            'description': f'The main web service for {settings.SITE_IDENTIFIER}.',
            'referenceURL': settings.SITE_URL.rstrip('/')
        },
        'capabilities': [
            {
                'interface': {
                    'subjects': ['Web browser'],
                    'type': 'vr:WebBrowser',
                    'access_url': {
                        'use': 'full',
                        'url': settings.SITE_URL.rstrip('/')
                    }
                }
            }
        ],
        'created': settings.SITE_CREATED,
        'updated': settings.SITE_UPDATED,
        'type': 'vs:CatalogResource',
        'status': 'active'
    }


def get_availability():
    return {
        'available': 'true',
        'note': 'service is accepting queries'
    }


def get_capabilities():
    return [
        {
            'id': 'ivo://ivoa.net/std/Registry',
            'type': 'vg:Harvest',
            'interface': {
                'type': 'vg:OAIHTTP',
                'role': 'std',
                'access_url': {
                    'use': 'base',
                    'url': settings.SITE_URL.rstrip('/') + reverse('oai:root')
                }
            },
            'max_records': settings.OAI_PAGE_SIZE
        },
        {
            'id': 'ivo://ivoa.net/std/VOSI#availability"',
            'interface': {
                'type': 'vs:ParamHTTP',
                'role': 'std',
                'access_url': {
                    'use': 'full',
                    'url': settings.SITE_URL.rstrip('/') + reverse('registry:availability')
                }
            }
        },
        {
            'id': 'ivo://ivoa.net/std/VOSI#capabilities',
            'interface': {
                'type': 'vs:ParamHTTP',
                'role': 'std',
                'access_url': {
                    'use': 'full',
                    'url': settings.SITE_URL.rstrip('/') + reverse('registry:capabilities')
                }
            }
        }
    ]


def get_tap_tableset():
    if apps.is_installed('daiquiri.tap'):
        from daiquiri.tap.vo import get_tableset
        return get_tableset()



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-03-11 16:57
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_auth', '0005_view_permission_fix'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='profile',
            options={'ordering': ('user__last_name', 'user__last_name', 'user__username'), 'permissions': (('view_profile', 'Can view Profile'),), 'verbose_name': 'Profile', 'verbose_name_plural': 'Profiles'},
        ),
    ]



# Generated by Django 2.1.4 on 2019-06-05 15:00

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Record',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('identifier', models.CharField(db_index=True, max_length=256, verbose_name='OAI identifier')),
                ('datestamp', models.DateField(db_index=True, verbose_name='OAI datestamp')),
                ('metadata_prefix', models.CharField(db_index=True, max_length=16, verbose_name='OAI metadataPrefix')),
                ('deleted', models.BooleanField(db_index=True, default=False, verbose_name='Deleted')),
            ],
            options={
                'verbose_name_plural': 'Records',
                'db_table': 'records',
                'ordering': ('-datestamp',),
                'verbose_name': 'Record',
            },
        ),
    ]



from django import forms
from django.conf import settings
from django.core.exceptions import BadRequest
from django.forms.renderers import TemplatesSetting


class DaiquiriFormRenderer(TemplatesSetting):
    form_template_name = 'core/form/form.html'


class HoneypotInput(forms.TextInput):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.attrs.update({
            'class': 'form-class', 'tabindex': '-1', 'autocomplete': 'off'
        })


class HoneypotField(forms.CharField):

    def __init__(self, *args, **kwargs):
        kwargs.setdefault('widget', HoneypotInput())
        kwargs['required'] = False
        kwargs['label'] = '' if settings.HONEYPOT_FIELD_HIDDEN else settings.HONEYPOT_FIELD_NAME
        super().__init__(*args, **kwargs)

    def validate(self, value):
        return value == settings.HONEYPOT_FIELD_VALUE

    def clean(self, value):
        cleaned_value = super().clean(value)
        if self.validate(cleaned_value) is True:
            return cleaned_value
        else:
            raise BadRequest('invalid request')



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-07-21 12:33
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_query', '0012_query_language_length'),
    ]

    operations = [
        migrations.AddField(
            model_name='example',
            name='access_level',
            field=models.CharField(choices=[(b'PRIVATE', 'Private - access must be granted by group'), (b'INTERNAL', 'Internal - logged in users can access'), (b'PUBLIC', 'Public - anonymous visitors can access')], default='PRIVATE', max_length=8, verbose_name='Access level'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='example',
            name='query_language',
            field=models.CharField(default='adql-2.0', help_text='The query language for this example.', max_length=16, verbose_name='Query language'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='example',
            name='description',
            field=models.TextField(blank=True, help_text='A brief description of the example to be displayed in the user interface.', null=True, verbose_name='Description'),
        ),
        migrations.AlterField(
            model_name='example',
            name='groups',
            field=models.ManyToManyField(blank=True, help_text='The groups which have access to the examples.', to='auth.Group', verbose_name='Groups'),
        ),
        migrations.AlterField(
            model_name='example',
            name='name',
            field=models.CharField(help_text='Identifier of the example.', max_length=256, verbose_name='Name'),
        ),
        migrations.AlterField(
            model_name='example',
            name='order',
            field=models.IntegerField(blank=True, help_text='Position in lists.', null=True, verbose_name='Order'),
        ),
        migrations.AlterField(
            model_name='example',
            name='query_string',
            field=models.TextField(help_text='The query string (SQL) for this example.', verbose_name='Query string'),
        ),
    ]



from django.contrib.auth.mixins import LoginRequiredMixin
from django.views.generic import TemplateView

from daiquiri.core.utils import get_model_field_meta
from daiquiri.core.views import AnonymousAccessMixin, CSRFViewMixin, ModelPermissionMixin, StoreIdViewMixin

from .models import Example, QueryJob


class QueryView(AnonymousAccessMixin, CSRFViewMixin, StoreIdViewMixin, TemplateView):
    template_name = 'query/query.html'
    anonymous_setting = 'QUERY_ANONYMOUS'


class JobsView(LoginRequiredMixin, CSRFViewMixin, TemplateView):
    template_name = 'query/jobs.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['phases'] = QueryJob.PHASE_CHOICES
        return context


class ExamplesView(ModelPermissionMixin, CSRFViewMixin, TemplateView):

    template_name = 'query/examples.html'
    permission_required = 'daiquiri_query.view_example'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['meta'] = {
            'Example': get_model_field_meta(Example)
        }
        return context


class NewJobsView(LoginRequiredMixin, CSRFViewMixin, StoreIdViewMixin, TemplateView):
    template_name = 'query/new/jobs.html'


class NewExamplesView(ModelPermissionMixin, CSRFViewMixin, StoreIdViewMixin, TemplateView):

    template_name = 'query/new/examples.html'
    permission_required = 'daiquiri_query.view_example'



# Generated by Django 3.2.12 on 2022-03-16 11:29

import daiquiri.core.encoders
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_stats', '0007_use_django_jsonfield'),
    ]

    operations = [
        migrations.AlterField(
            model_name='record',
            name='resource',
            field=models.JSONField(encoder=daiquiri.core.encoders.UUIDJSONEncoder),
        ),
    ]



from django.urls import include, re_path
from django.views.generic import TemplateView

from .views import logout, password_change, password_set, profile_json, profile_update, terms_of_use, token

urlpatterns = [
    re_path(r'^profile/$', profile_update, name='account_profile'),
    re_path(r'^profile.json/$', profile_json, name='account_profile_json'),
    re_path(r'^token/$', token, name='account_token'),
    re_path(r'^pending/$', TemplateView.as_view(template_name='account/account_pending.html'), name='account_pending'),

    re_path(r"^logout/$", logout, name="account_logout"),

    # include allauth patterns
    re_path(r'^password/change/$', password_change, name='account_change_password'),
    re_path(r'^password/change/done/$', TemplateView.as_view(template_name='account/password_change_done.html'), name='account_change_password_done'),  # noqa: E501
    re_path(r'^password/set/$', password_set, name='account_password_set'),
    re_path(r'^password/set/done/$', TemplateView.as_view(template_name='account/password_set_done.html'), name='account_set_password_done'),  # noqa: E501
    re_path(r'^terms_of_use/$', terms_of_use, name='terms_of_use'),
    re_path(r'^', include('allauth.urls')),
]



import os

from django.conf import settings
from django.contrib.auth.models import Group
from django.db import models
from django.utils.translation import gettext_lazy as _

from daiquiri.core.constants import ACCESS_LEVEL_CHOICES
from daiquiri.core.managers import AccessLevelManager


class Directory(models.Model):

    path = models.CharField(
        max_length=256, blank=True,
        verbose_name=_('Path'),
        help_text=_('Path of the directory.')
    )
    layout = models.BooleanField(
        verbose_name=_('Layout'), default=True,
        help_text=_('Use the page layout with the content.')
    )
    access_level = models.CharField(
        max_length=8, choices=ACCESS_LEVEL_CHOICES,
        verbose_name=_('Access level')
    )
    groups = models.ManyToManyField(
        Group, blank=True,
        verbose_name=_('Groups'),
        help_text=_('The groups which have access to this function.')
    )
    depth = models.IntegerField(default=0)

    objects = AccessLevelManager()

    class Meta:
        ordering = ('path', )

        verbose_name = _('Directory')
        verbose_name_plural = _('Directory')

    def __str__(self):
        return self.absolute_path

    def save(self):
        self.depth = len(os.path.normpath(self.path).split(os.path.sep))
        super().save()

    @property
    def absolute_path(self):
        return os.path.normpath(os.path.join(settings.FILES_BASE_PATH, os.path.normpath(self.path)))



import logging

from .adapter import DatalinkAdapter
from .models import Datalink

logger = logging.getLogger(__name__)


def update_links(resource_type, resource):
    logger.debug('update_links %s %s', resource_type, resource)

    adapter = DatalinkAdapter()

    if resource_type in adapter.resource_types:
        identifier = adapter.get_identifier(resource_type, resource)
        links = adapter.get_links(resource_type, resource)

        datalinks = []
        for link in links:
            datalinks.append(Datalink(**link))

        Datalink.objects.filter(ID=identifier).delete()
        Datalink.objects.bulk_create(datalinks)


def delete_links(resource_type, resource):
    logger.debug('delete_links %s %s', resource_type, resource)

    adapter = DatalinkAdapter()

    if resource_type in adapter.resource_types:
        identifier = adapter.get_identifier(resource_type, resource)

        Datalink.objects.filter(ID=identifier).delete()



class DataciteRendererMixin:

    def render_datacite(self, metadata):
        self.start('resource', {
            'xmlns': 'http://datacite.org/schema/kernel-4',
            'xmlns:xsi': 'http://www.w3.org/2001/XMLSchema-instance',
            'xsi:schemaLocation': 'http://datacite.org/schema/kernel-4 http://schema.datacite.org/meta/kernel-4.4/metadata.xsd'
        })
        self.node('identifier', {'identifierType': 'DOI'}, metadata.get('identifier'))

        self.start('creators')
        creators = metadata.get('creators', [])
        if isinstance(creators, list):
            for creator in creators:
                self.render_person('creator', creator)
        self.end('creators')

        self.start('titles')
        self.node('title', {'xml:lang': 'en'}, metadata.get('title'))
        self.end('titles')

        self.node('publisher', {}, metadata.get('publisher'))
        self.node('publicationYear', {}, metadata.get('publication_year'))

        self.start('subjects')
        for subject in metadata.get('subjects', []):
            self.node('subject', {
                'xml:lang': 'en',
                'subjectScheme': subject.get('subjectScheme'),
                'schemeURI': subject.get('schemeURI'),
                'valueURI': subject.get('valueURI')
            }, subject.get('subject'))
        self.end('subjects')

        self.start('contributors')
        contributors = metadata.get('contributors', [])
        if isinstance(contributors, list):
            for contributor in contributors:
                self.render_person('contributor', contributor, contributor.get('contributor_type', 'DataManager'))
        self.end('contributors')

        updated = metadata.get('updated')
        published = metadata.get('published')
        if updated is not None or published is not None:
            self.start('dates')
            if updated:
                self.node('date', {'dateType': 'Updated'}, updated)
            if published:
                self.node('date', {'dateType': 'Issued'}, published)
            self.end('dates')

        self.node('language', {}, metadata.get('language'))
        self.node('resourceType', {'resourceTypeGeneral': 'Dataset'}, metadata.get('resource_type'))

        related_identifiers = metadata.get('related_identifiers')
        if related_identifiers is not None:
            self.start('relatedIdentifiers')
            for related_identifier in related_identifiers:
                self.node('relatedIdentifier', {
                    'relatedIdentifierType': related_identifier.get('related_identifier_type'),
                    'relationType': related_identifier.get('relation_type')
                }, related_identifier.get('related_identifier'))
            self.end('relatedIdentifiers')

        alternate_identifiers = metadata.get('alternate_identifiers')
        if alternate_identifiers is not None:
            self.start('alternateIdentifiers')
            for alternate_identifier in alternate_identifiers:
                self.node('alternateIdentifier', {
                    'alternateIdentifierType': alternate_identifier.get('alternate_identifier_type')
                }, alternate_identifier.get('alternate_identifier'))
            self.end('alternateIdentifiers')

        sizes = metadata.get('sizes')
        if sizes is not None:
            self.start('sizes')
            for size in sizes:
                self.node('size', {}, size)
            self.end('sizes')

        formats = metadata.get('formats')
        if formats is not None:
            self.start('formats')
            for format in formats:
                self.node('format', {}, format)
            self.end('formats')

        version = metadata.get('version')
        if version is not None:
            self.node('version', {}, version)

        license = metadata.get('license')
        if license is not None:
            self.start('rightsList')
            self.node('rights', {
                'rightsURI': metadata.get('license_url'),
                'rightsIdentifier': metadata.get('license_identifier'),
                },
                metadata.get('license_label'))
            self.end('rightsList')

        description = metadata.get('long_description') or metadata.get('description')
        if description is not None:
            self.start('descriptions')
            self.node('description', {
                'xml:lang': metadata.get('language'),
                'descriptionType': 'Abstract'
            }, description)
            self.end('descriptions')

        self.end('resource')

    def render_person(self, person_type, person, contributor_type=None):
        if isinstance(person, dict):
            self.start(person_type, {
                'contributorType': contributor_type
            })

            name = person.get('name')
            first_name = person.get('first_name')
            last_name = person.get('last_name')
            name_type = person.get('name_type')

            if not name:
                name = f'{last_name}, {first_name}'

            self.node(person_type + 'Name', {'nameType': name_type}, name)

            if first_name:
                self.node('givenName', {}, first_name)

            if last_name:
                self.node('familyName', {}, last_name)

            orcid = person.get('orcid')
            if orcid:
                self.node('nameIdentifier', {
                    'schemeURI': 'http://orcid.org/',
                    'nameIdentifierScheme': 'ORCID'
                }, orcid)

            ror = person.get('ror')
            if ror:
                self.node('nameIdentifier', {
                    'schemeURI': 'http://ror.org/',
                    'nameIdentifierScheme': 'ROR'
                }, ror)

            affiliations = person.get('affiliations')
            if affiliations:
                for affiliation in affiliations:
                    self.node('affiliation', {
                        'affiliationIdentifier': affiliation.get('affiliation_identifier'),
                        'affiliationIdentifierScheme': affiliation.get('affiliation_identifier_scheme'),
                        'schemeURI': affiliation.get('scheme_uri')
                        }, affiliation.get('affiliation'))

            self.end(person_type)



from django.apps import AppConfig
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured


class FilesConfig(AppConfig):
    name = 'daiquiri.files'
    label = 'daiquiri_files'
    verbose_name = 'Files'

    def ready(self):
        if not settings.FILES_BASE_PATH:
            raise ImproperlyConfigured('FILES_BASE_PATH is not set')



# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2017-12-09 14:59
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_jobs', '0012_meta'),
        ('daiquiri_query', '0013_refactoring'),
    ]

    operations = [
        migrations.CreateModel(
            name='DownloadJob',
            fields=[
                ('job_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='daiquiri_jobs.Job')),
                ('format_key', models.CharField(help_text='Format key for this download.', max_length=32, verbose_name='Format key')),
                ('file_path', models.CharField(help_text='Path to the file.', max_length=256, verbose_name='Path')),
                ('job', models.ForeignKey(help_text='QueryJob this DownloadJob belongs to.', on_delete=django.db.models.deletion.CASCADE, related_name='downloads', to='daiquiri_query.QueryJob', verbose_name='QueryJob')),
            ],
            options={
                'ordering': ('start_time',),
                'verbose_name': 'DownloadJob',
                'verbose_name_plural': 'DownloadJobs',
                'permissions': (('view_downloadjob', 'Can view DownloadJob'),),
            },
            bases=('daiquiri_jobs.job',),
        ),
    ]



from django.core.exceptions import ImproperlyConfigured

from rest_framework.permissions import BasePermission, DjangoModelPermissions


class HasModelPermission(DjangoModelPermissions):

    perms_map = {
        'GET': ['%(app_label)s.view_%(model_name)s'],
        'OPTIONS': ['%(app_label)s.view_%(model_name)s'],
        'HEAD': ['%(app_label)s.view_%(model_name)s'],
        'POST': ['%(app_label)s.add_%(model_name)s'],
        'PUT': ['%(app_label)s.change_%(model_name)s'],
        'PATCH': ['%(app_label)s.change_%(model_name)s'],
        'DELETE': ['%(app_label)s.delete_%(model_name)s'],
    }


class HasObjectPermission(BasePermission):

    perms_map = {
        'GET': 'view',
        'OPTIONS': 'view',
        'HEAD': 'view',
        'POST': 'add',
        'PUT': 'change',
        'PATCH': 'change',
        'DELETE': 'delete'
    }

    def get_permission_required(self, view, method):
        '''
        inspired by django.contrib.auth.mixins.PermissionRequiredMixin
        '''
        if view.permission_required is None:
            raise ImproperlyConfigured(f'{view.__class__.__name__} is missing the permission_required attribute.')

        permission_required = view.permission_required[self.perms_map[method]]

        if isinstance(permission_required, str):
            perms = (permission_required, )
        else:
            perms = permission_required
        return perms

    def has_permission(self, request, view):
        try:
            obj = view.get_permission_object()
        except AttributeError:
            obj = None

        perms = self.get_permission_required(view, request.method)

        if obj:
            return request.user.has_perms(perms, obj)
        else:
            return request.user.has_perms(perms)

    def has_object_permission(self, request, view, obj):
        try:
            obj = view.get_permission_object()
        except AttributeError:
            pass

        perms = self.get_permission_required(view, request.method)
        return request.user.has_perms(perms, obj)



import hashlib

from django.conf import settings
from django.contrib.auth.mixins import AccessMixin
from django.contrib.auth.mixins import PermissionRequiredMixin as DjangoPermissionRequiredMixin
from django.contrib.auth.views import redirect_to_login
from django.core.exceptions import PermissionDenied
from django.shortcuts import render
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import ensure_csrf_cookie
from django.views.generic import View

from allauth.account.forms import LoginForm
from rules.contrib.views import PermissionRequiredMixin as RulesPermissionRequiredMixin

from daiquiri.core.utils import render_to_xml


def home(request):
    if not request.user.is_authenticated:
        login_form = LoginForm()
        login_form.fields['login'].widget.attrs.pop("autofocus", None)
    else:
        login_form = None

    return render(request, 'core/home.html', {'form': login_form})


def bad_request(request, exception):
    return render(request, 'core/400.html', status=400)


def forbidden(request, exception):
    return render(request, 'core/403.html', status=403)


def not_found(request, exception):
    return render(request, 'core/404.html', status=404)


def internal_server_error(request):
    return render(request, 'core/500.html', status=500)


class PermissionRedirectMixin:

    def handle_no_permission(self):
        if self.request.user.is_authenticated:
            raise PermissionDenied(self.get_permission_denied_message())

        return redirect_to_login(self.request.get_full_path(), self.get_login_url(), self.get_redirect_field_name())


class ModelPermissionMixin(PermissionRedirectMixin, DjangoPermissionRequiredMixin):
    pass


class ObjectPermissionMixin(PermissionRedirectMixin, RulesPermissionRequiredMixin):
    pass


class AnonymousAccessMixin(AccessMixin):
    anonymous_setting = None

    def dispatch(self, request, *args, **kwargs):
        if not getattr(settings, self.anonymous_setting) and not request.user.is_authenticated:
            return self.handle_no_permission()
        return super().dispatch(request, *args, **kwargs)


class SingleObjectXMLMixin(View):

    def get(self, request, *args, **kwargs):
        self.object = self.get_object()

        context = self.get_context_data()
        renderer = self.renderer_class()

        return render_to_xml(request, renderer, context)

    def get_object(self):
        raise NotImplementedError()

    def get_context_data(self):
        raise NotImplementedError()


class CSRFViewMixin(View):

    @method_decorator(ensure_csrf_cookie)
    def get(self, request, *args, **kwargs):
        return super().get(self, request, *args, **kwargs)


class StoreIdViewMixin(View):

    def render_to_response(self, context, **response_kwargs):
        response = super().render_to_response(context, **response_kwargs)
        response.set_cookie('storeid', self.get_store_id(), samesite='Lax')
        return response

    def get_store_id(self):
        session_key = self.request.session.session_key or 'anonymous'
        return hashlib.sha256(session_key.encode()).hexdigest()



# -*- coding: utf-8 -*-
# Generated by Django 1.11.10 on 2018-02-16 17:22
from __future__ import unicode_literals
from django.conf import settings

from django.db import migrations, models, connection


def fix_index_name(apps, schema_editor):
    # fix wrong primary key index and sequence name since
    # django does not do it correctly
    # see https://code.djangoproject.com/ticket/23577

    if settings.DATABASES['default']['ENGINE'] == 'django.db.backends.postgresql':
        with connection.cursor() as cursor:
            cursor.execute('ALTER INDEX daiquiri_metadata_database_pkey RENAME TO daiquiri_metadata_schema_pkey;')
            cursor.execute('ALTER SEQUENCE daiquiri_metadata_database_id_seq RENAME TO daiquiri_metadata_schema_id_seq;')


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_metadata', '0016_rename_database_to_schema'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='Database',
            new_name='Schema',
        ),
        migrations.AlterModelOptions(
            name='schema',
            options={'ordering': ('order',), 'permissions': (('view_schema', 'Can view Schema'),), 'verbose_name': 'Schema', 'verbose_name_plural': 'Schemas'},
        ),
        migrations.RunPython(fix_index_name)
    ]



'''
Runs daiquiri-admin when the daiquiri module is run as a script, much like django-admin
(see: https://github.com/django/django/blob/main/django/__main__.py):

    python -m daiquiri check

The main method is added as script in pyproject.toml so that

    daiquiri-admin check

works as well. Unlike django-admin, a set of generic settings is used for the
management scripts.
'''

import os

from django.core import management

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "daiquiri.core.management.settings")


def main():
    management.execute_from_command_line()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-07-21 12:33
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_metadata', '0007_access_level'),
    ]

    operations = [
        migrations.AlterField(
            model_name='column',
            name='access_level',
            field=models.CharField(choices=[(b'PRIVATE', 'Private - access must be granted by group'), (b'INTERNAL', 'Internal - logged in users can access'), (b'PUBLIC', 'Public - anonymous visitors can access')], default='PRIVATE', max_length=8, verbose_name='Access level'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='column',
            name='metadata_access_level',
            field=models.CharField(choices=[(b'PRIVATE', 'Private - access must be granted by group'), (b'INTERNAL', 'Internal - logged in users can access'), (b'PUBLIC', 'Public - anonymous visitors can access')], default='PRIVATE', max_length=8, verbose_name='Metadata access level'),
            preserve_default=False,
        ),
    ]
